;*****************************************************************************
;				NICST TOOL
;
;  RC-5 Part 2 test for TEB bands. Calculates the SNR and NEdT. Fits the SNR
;  to the OBC detla radiance (using a function with Powell's method) 
;  with or without weights of 1/std. This is done on a scan by scan basis.
;
;  Input:
;  1) Input file.
;
;  Output:
;  1) SNR[collect,band,HAM,detector,subsample]
;  2) snr_1[collect,band,HAM,detector,subsample,scan]
;  3) snr_coeff[band,HAM,detector,subsample,coefficients,fit type,not weighted or weighted]
;  4) snr_fit[collect,band,HAM,detector,subsample,scan,fit type,not weighted or weighted]
;  5) NEdL[collect,b,HAM,detector,subsample,scan,fit type,not weighted or weighted]
;  6) NEdT[collect,band,HAM,detector,subsample,scan,fit type,not weighted or weighted]
;
;  v1.0 - 04/3/2008 - Jeff McIntire
;*****************************************************************************

;*****************************************************************************
;  Functions for powell's method.
;*****************************************************************************

FUNCTION powfunc, xx

COMMON X1,b
COMMON X2,id
COMMON X3,ih
COMMON X4,is
COMMON X5,ncollects
COMMON X6,SNR_1
COMMON X7,rad_OBC_scan_x
COMMON X8,lcol
COMMON X9,hcol
COMMON X10,w
COMMON X11,iscan
COMMON X12,fnum

X=fltarr(ncollects)

Z=0
FOR col=lcol, hcol-1 DO BEGIN
	IF (xx[0]+xx[1]*rad_OBC_scan_x[col,b,ih,id,is,iscan]*fnum+xx[2]*(rad_OBC_scan_x[col,b,ih,id,is,iscan]*fnum)^2) LT 0 $
		THEN RETURN,1000000
	X[col]=rad_OBC_scan_x[col,b,ih,id,is,iscan]*fnum-SNR_1[col,b,ih,id,is,iscan] $
		*sqrt(xx[0]+xx[1]*rad_OBC_scan_x[col,b,ih,id,is,iscan]*fnum+xx[2]*(rad_OBC_scan_x[col,b,ih,id,is,iscan]*fnum)^2)

	Z=Z+w[col]*X[col]^2/total(w)
ENDFOR

RETURN,Z

END

;*****************************************************************************
;  Main program.
;*****************************************************************************

PRO RC5_OBC_SNR_NEdT, infile

nlines=File_Lines(infile)
header=strarr(1)
inpath=strarr(1)		; Input path.
outpath=strarr(1)		; Output path.
rvs_in=strarr(1)
param_in=strarr(1)
rsrpath=strarr(1)
fnumber=fltarr(1)

openr,lun,infile,/Get_Lun	
readf,lun,header	
readf,lun,inpath
readf,lun,outpath
readf,lun,rvs_in
readf,lun,param_in
readf,lun,rsrpath
readf,lun,fnumber
Free_Lun,lun

a2=strsplit(outpath,'/',/Extract)
a3=N_ELEMENTS(a2)

infname1=outpath+a2[a3-3]+'.'+a2[a3-2]+'.'+a2[a3-1]+'.dn.sav'
infname2=outpath+a2[a3-3]+'.'+a2[a3-2]+'.'+a2[a3-1]+'.rad.sav'
infname3=outpath+a2[a3-3]+'.'+a2[a3-2]+'.'+a2[a3-1]+'.temp.sav'
restore,infname1
restore,infname2
restore,infname3

restore,'general.TEB.sav'

outfile=outpath+a2[a3-3]+'.'+a2[a3-2]+'.'+a2[a3-1]+'.snr.sav'

COMMON X1,b
COMMON X2,id
COMMON X3,ih
COMMON X4,is
COMMON X5,ncollects
COMMON X6,SNR_1
COMMON X7,rad_OBC_scan_x
COMMON X8,lcol
COMMON X9,hcol
COMMON X10,w
COMMON X11,iscan
COMMON X12,fnum

fnum=fnumber

;*****************************************************************************
;  Reform the radiance to get rid of extra zeros.
;*****************************************************************************

maxscan=100
dn_x=dblarr(ncollects,nbands_TEB,nham,maxdet,maxsubsets,maxscan)
std_x=dblarr(ncollects,nbands_TEB,nham,maxdet,maxsubsets,maxscan)
rad_OBC_scan_x=dblarr(ncollects,nbands_TEB,nham,maxdet,maxsubsets,maxscan)
FOR b=0, nbands_TEB-1 DO BEGIN
	FOR is=0, subsets_TEB[b]-1 DO BEGIN
		FOR ih=0, nham-1 DO BEGIN
			FOR id=0, detectors_TEB[b]-1 DO BEGIN
				FOR col=0, ncollects-1 DO BEGIN
					index1=WHERE(dn_col[col,b,ih,id,is,1,*,2] NE 0, arr1)
					index2=WHERE(std_col[col,b,ih,id,is,1,*,2] NE 0, arr2)
					index3=WHERE(abs(rad_OBC_scan[col,b,ih,id,is,*]) GT 0.01, arr3)
					dn_x[col,b,ih,id,is,0:49]=dn_col[col,b,ih,id,is,1,index1,2]
					std_x[col,b,ih,id,is,0:49]=std_col[col,b,ih,id,is,1,index2,2]
					rad_OBC_scan_x[col,b,ih,id,is,0:49]=rad_OBC_scan[col,b,ih,id,is,index3]*0.4444
				ENDFOR
			ENDFOR
		ENDFOR
	ENDFOR
ENDFOR

;*****************************************************************************
;  Calculate the SNR.
;*****************************************************************************

SNR_1=dblarr(ncollects,nbands_TEB,nham,maxdet,maxsubsets,maxscan/2)
SNR_2=dblarr(ncollects,nbands_TEB,nham,maxdet,maxsubsets,maxscan/2)
SNR=dblarr(ncollects,nbands_TEB,nham,maxdet,maxsubsets)
sigma_SNR=dblarr(ncollects,nbands_TEB,nham,maxdet,maxsubsets)

FOR col=0, ncollects-1 DO BEGIN
	FOR b=0, nbands_TEB-1 DO BEGIN
		IF band_type_TEB[b] EQ 1 THEN max_frame=400
		IF band_type_TEB[b] EQ 0 THEN max_frame=200
		FOR ih=0, nham-1 DO BEGIN
			FOR is=0, subsets_TEB[b]-1 DO BEGIN
				FOR id=0, detectors_TEB[b]-1 DO BEGIN
					SNR_1[col,b,ih,id,is,0:maxscan/2-1]=dn_x[col,b,ih,id,is,0:maxscan/2-1] $
						/std_x[col,b,ih,id,is,0:maxscan/2-1]

					SNR[col,b,ih,id,is]=sig_mean(SNR_1[col,b,ih,id,is,0:maxscan/2-1],3,1)
					FOR iscan=0, maxscan/2-1 DO BEGIN
						SNR_2[col,b,ih,id,is,iscan]=(SNR_1[col,b,ih,id,is,iscan]-SNR[col,b,ih,id,is])^2
					ENDFOR

					sigma_SNR[col,b,ih,id,is]=sqrt(sig_mean(SNR_2[col,b,ih,id,is,0:maxscan/2-1],3,1))
				ENDFOR
			ENDFOR
		ENDFOR
	ENDFOR
ENDFOR

;*****************************************************************************
;  Powell's method and ploy_fit (cubic) for fitting SNR to BCS radiance.
;*****************************************************************************

c=dblarr(ncollects)
d=dblarr(ncollects)
w=dblarr(ncollects)

snr_coeff=dblarr(nbands_TEB,nham,maxdet,maxsubsets,maxscan,4,2,2)
snr_fit=dblarr(ncollects,nbands_TEB,nham,maxdet,maxsubsets,maxscan,2,2)

FOR ifit=0, 1 DO BEGIN
	FOR b=0, nbands_TEB-1 DO BEGIN
		FOR is=0, subsets_TEB[b]-1 DO BEGIN
			FOR ih=0, nham-1 DO BEGIN
				FOR id=0, detectors_TEB[b]-1 DO BEGIN
					IF ifit EQ 0 THEN w[*]=1.0
					IF ifit EQ 1 THEN w[*]=1/sigma_SNR[*,b,ih,id,is]^2

					FOR iscan=0, maxscan/2-1 DO BEGIN

						; Warm up collects.
						lcol=0
						hcol=ncollects
						FOR col=0, ncollects-2 DO BEGIN
							IF T_obc[ih,col] GT T_obc[ih,col+1] THEN hcol=hcol-1
						ENDFOR

						P1=[.000001,.000001,.000001]
						ftol=0.00000001
						xi=TRANSPOSE([[0.0,1.0,0.0],[0.0,0.0,1.0],[1.0,0.0,0.0]])

						powell,P1,xi,ftol,fmin,'powfunc',/DOUBLE,ITMAX=1000
						snr_coeff[b,ih,id,is,iscan,0:2,0,ifit]=P1[0:2]

						; Cool down collects.
						lcol=hcol
						hcol=ncollects

						P2=[.0001,.0001,.0001]
						ftol=0.00000001
						xi=TRANSPOSE([[0.0,1.0,0.0],[0.0,0.0,1.0],[1.0,0.0,0.0]])

						powell,P2,xi,ftol,fmin,'powfunc',/DOUBLE,ITMAX=1000
						snr_coeff[b,ih,id,is,iscan,0:2,1,ifit]=P2[0:2]

					;	print,ifit,b,id,ih,is,iscan,P1[0],P1[1],P1[2],fmin,P2[0],P2[1],P2[2]

						FOR col=0, ncollects-1 DO BEGIN
							snr_fit[col,b,ih,id,is,iscan,0,ifit]=(SNR_1[col,b,ih,id,is,iscan] $
								-(rad_OBC_scan_x[col,b,ih,id,is,iscan]*fnumber) $
								/sqrt(P1[0]+P1[1] $
								*(rad_OBC_scan_x[col,b,ih,id,is,iscan]*fnumber) $
								+P1[2]*(rad_OBC_scan_x[col,b,ih,id,is,iscan]*fnumber)^2)) $
								/SNR_1[col,b,ih,id,is,iscan]
							snr_fit[col,b,ih,id,is,iscan,1,ifit]=(SNR_1[col,b,ih,id,is,iscan] $
								-(rad_OBC_scan_x[col,b,ih,id,is,iscan]*fnumber) $
								/sqrt(P2[0]+P2[1] $
								*(rad_OBC_scan_x[col,b,ih,id,is,iscan]*fnumber) $
								+P2[2]*(rad_OBC_scan_x[col,b,ih,id,is,iscan]*fnumber)^2)) $
								/SNR_1[col,b,ih,id,is,iscan]
						ENDFOR
					ENDFOR
				ENDFOR
			ENDFOR
		ENDFOR
	ENDFOR
ENDFOR

;*****************************************************************************
;  Calulate the NEdL and NEdT.
;*****************************************************************************

NEdL=dblarr(ncollects,nbands_TEB,nham,maxdet,maxsubsets,maxscan/2,2,2)
NEdT=dblarr(ncollects,nbands_TEB,nham,maxdet,maxsubsets,maxscan/2,2,2)

FOR ifit1=0, 1 DO BEGIN
	FOR b=0, nbands_TEB-1 DO BEGIN
		FOR col=0, ncollects-1 DO BEGIN
			FOR ifit2=0, 1 DO BEGIN
				NEdL[col,b,0:1,0:detectors_TEB[b]-1,0:subsets_TEB[b]-1,0:maxscan/2-1,ifit2,ifit1] $
					=rad_OBC_scan_x[col,b,0:1,0:detectors_TEB[b]-1,0:subsets_TEB[b]-1,0:maxscan/2-1] $
					/SNR_1[col,b,0:1,0:detectors_TEB[b]-1,0:subsets_TEB[b]-1,0:maxscan/2-1]
				NEdT[col,b,0:1,0:detectors_TEB[b]-1,0:subsets_TEB[b]-1,0:maxscan/2-1,ifit2,ifit1] $
					=NEdL[col,b,0:1,0:detectors_TEB[b]-1,0:subsets_TEB[b]-1,0:maxscan/2-1,ifit2,ifit1] $
					/Planck_deriv(T_bcs[col],lambda_TEB[b])
			ENDFOR
		ENDFOR
	ENDFOR
ENDFOR

save,/compress,SNR_1,SNR,snr_coeff,snr_fit,NEdL,NEdT,filename=outfile

END
