;*****************************************************************************
;				NICST TOOL
;
;  RC-5 Part 1 test for TEB bands. Calculates the effective EV source radiance
;  and compares to specifications.
;
;  Input:
;  1) Input file with paths to RVS, and general info sav files as well
;     as the .temp.sav, .dn.sav, .rad.sav, .rg.sav, and .snr.sav files.
;
;  Output:
;  1) rad_OBC_ret[collect,band,detector,ifit]
;  2) max_ARD_obc[collect,band,HAM,subsample,ifit]
;  3) emiss_OBC[collect,band,ifit]
;  
;  v1.0 - 06/27/2008 - Jeff McIntire
;*****************************************************************************

PRO RC5_OBC_eff_rad, infile

;*****************************************************************************
;  Read in input file and sav TEB file.
;*****************************************************************************

nlines=File_Lines(infile)
ab=''
header=strarr(1)
inpath=strarr(1)		; Input path.
outpath=strarr(1)		; Output path.
rvs_in=strarr(1)
param_in=strarr(1)
rsrpath=strarr(1)
fnumber=strarr(1)
header_size=intarr(1)

openr,lun,infile,/Get_Lun	
readf,lun,header	
readf,lun,inpath
readf,lun,outpath
readf,lun,rvs_in
readf,lun,param_in
readf,lun,rsrpath
readf,lun,fnumber
readf,lun,header_size
Free_Lun,lun
Free_Lun,lun

ham=initial_ham_side(inpath,header_size)

a2=strsplit(outpath,'/',/Extract)
a3=N_ELEMENTS(a2)

infname1=outpath+a2[a3-3]+'.'+a2[a3-2]+'.'+a2[a3-1]+'.dn.sav'
infname2=outpath+a2[a3-3]+'.'+a2[a3-2]+'.'+a2[a3-1]+'.rad.sav'
infname3=outpath+a2[a3-3]+'.'+a2[a3-2]+'.'+a2[a3-1]+'.temp.sav'
infname4=outpath+a2[a3-3]+'.'+a2[a3-2]+'.'+a2[a3-1]+'.rg.sav'
restore,infname1
restore,infname2
restore,infname3
restore,infname4

restore,'general.TEB.sav'

outfile=outpath+a2[a3-3]+'.'+a2[a3-2]+'.'+a2[a3-1]+'.eff_rad.sav'

NEDT_spec=[[2.5,1.5,0.396,0.107,0.091,0.07,0.072,0.072],[0,0,0,0.423,0,0,0,0]]
T_typ_TEB_f9=[[286.,243.,286.,322.,310.,340.,340.,340.],[0,0,0,416.,0,0,0,0]]

mindet=[0,0,0,0,1,0,0,0]		; Eliminate detector 1 band M14.

;*****************************************************************************
;  Reform the dn and radiance to get rid of extra zeros (use the OBC radiance 
;  to get the right indices for the BCS radiance).
;*****************************************************************************

max_scan=100

dn_x=dblarr(ncollects,nbands_TEB,nham,maxdet,maxsubsets,max_scan)
dn_y=dblarr(ncollects,nbands_TEB,nham,maxdet,maxsubsets,max_scan)
rad_BCS_scan_x=dblarr(ncollects,nbands_TEB,nham,maxdet,maxsubsets,max_scan)
rad_p3_x=dblarr(ncollects,nbands_TEB,nham,maxdet,max_scan,5)

FOR b=0, nbands_TEB-1 DO BEGIN
	FOR is=0, subsets_TEB[b]-1 DO BEGIN
		FOR ih=0, nham-1 DO BEGIN
			FOR id=0, detectors_TEB[b]-1 DO BEGIN
				FOR col=0, ncollects-1 DO BEGIN
					index1=WHERE(dn_col[col,b,ih,id,is,1,*,0] NE 0, arr1)
					index2=WHERE(dn_col[col,b,ih,id,is,1,*,2] NE 0, arr2)
					index3=WHERE(abs(rad_OBC_scan[col,b,ih,id,is,*]) GT 0.01, arr3)
					index4=WHERE(abs(rad_p3[col,b,ih,id,*,0]) NE 0, arr4)
					index5=WHERE(abs(rad_p3[col,b,ih,id,*,1]) NE 0, arr5)
					index5=WHERE(abs(rad_p3[col,b,ih,id,*,4]) NE 0, arr6)
					dn_x[col,b,ih,id,is,0:49]=dn_col[col,b,ih,id,is,1,index1,0]
					dn_y[col,b,ih,id,is,0:49]=dn_col[col,b,ih,id,is,1,index2,2]
					rad_BCS_scan_x[col,b,ih,id,is,0:49]=rad_BCS_scan[col,b,ih,id,is,index3]
					rad_p3_x[col,b,ih,id,0:49,0]=rad_p3[col,b,ih,id,index4,0]
					rad_p3_x[col,b,ih,id,0:49,1]=rad_p3[col,b,ih,id,index5,1]
					rad_p3_x[col,b,ih,id,0:49,4]=rad_p3[col,b,ih,id,index5,4]
				ENDFOR
			ENDFOR
		ENDFOR
	ENDFOR
ENDFOR

;*****************************************************************************
;  Find retrived OBC radiances for RC5 part 2.
;*****************************************************************************

rad_OBC_ret_2=dblarr(ncollects,nbands_TEB,nham,maxdet,maxsubsets,max_scan/2,6)
ARD_obc_2=dblarr(ncollects,nbands_TEB,nham,maxdet,maxsubsets,max_scan/2,6)
ARD_obc=dblarr(ncollects,nbands_TEB,nham,maxdet,maxsubsets,6)
max_ARD_obc=dblarr(ncollects,nbands_TEB,nham,maxsubsets,6)
rad_OBC_ret=dblarr(ncollects,nbands_TEB,maxdet,6)
rad_OBC_bkg_2=dblarr(ncollects,nbands_TEB,nham,maxdet,maxsubsets,max_scan/2)
rad_OBC_bkg_1=dblarr(ncollects,nbands_TEB,nham,maxdet,maxsubsets)
rad_OBC_bkg=dblarr(ncollects,nbands_TEB,maxdet)
emiss_OBC_1=dblarr(ncollects,nbands_TEB,maxdet,6)
emiss_OBC=dblarr(ncollects,nbands_TEB,6)

FOR iscan=0, max_scan/2-1 DO BEGIN
	FOR b=0, nbands_TEB-1 DO BEGIN
		FOR ih=0, nham-1 DO BEGIN
			FOR is=0, subsets_TEB[b]-1 DO BEGIN
				FOR col=0, ncollects-1 DO BEGIN
					FOR id=0, detectors_TEB[b]-1 DO BEGIN
						rad_OBC_ret_2[col,b,ih,id,is,iscan,1] $
							=(rg_1[b,ih,id,is,iscan,0,1] $
							+rg_1[b,ih,id,is,iscan,1,1]*dn_y[col,b,ih,id,is,iscan] $
							+rg_1[b,ih,id,is,iscan,2,1]*dn_y[col,b,ih,id,is,iscan]^2) $
							/(rvs[b,is,id,ih,2]*p[b]) $
							+(rvs[b,is,id,ih,1]/rvs[b,is,id,ih,2])*rad_p1[col,b,id,1] $
							-(rvs[b,is,id,ih,1]-rvs[b,is,id,ih,2])/(rvs[b,is,id,ih,2]*p[b]) $
							*(rad_p3_x[col,b,ih,id,iscan,0]-(1-p[b])*rad_p3_x[col,b,ih,id,iscan,1])
						rad_OBC_ret_2[col,b,ih,id,is,iscan,2] $
							=(rg_1[b,ih,id,is,iscan,0,2] $
							+rg_1[b,ih,id,is,iscan,1,2]*dn_y[col,b,ih,id,is,iscan] $
							+rg_1[b,ih,id,is,iscan,2,2]*dn_y[col,b,ih,id,is,iscan]^2 $
							+rg_1[b,ih,id,is,iscan,3,2]*dn_y[col,b,ih,id,is,iscan]^3) $
							/(rvs[b,is,id,ih,2]*p[b]) $
							+(rvs[b,is,id,ih,1]/rvs[b,is,id,ih,2])*rad_p1[col,b,id,1] $
							-(rvs[b,is,id,ih,1]-rvs[b,is,id,ih,2])/(rvs[b,is,id,ih,2]*p[b]) $
							*(rad_p3_x[col,b,ih,id,iscan,0]-(1-p[b])*rad_p3_x[col,b,ih,id,iscan,1])
						rad_OBC_ret_2[col,b,ih,id,is,iscan,4] $
							=(rg_1[b,ih,id,is,iscan,0,1] $
							+rg_1[b,ih,id,is,iscan,1,1]*dn_y[col,b,ih,id,is,iscan] $
							+rg_1[b,ih,id,is,iscan,2,1]*dn_y[col,b,ih,id,is,iscan]^2) $
							/rvs[b,is,id,ih,2]/p[b]*(rad_BCS[col,b,ih,id,is]/(rg_1[b,ih,id,is,iscan,0,1] $
							+rg_1[b,ih,id,is,iscan,1,1]*dn_y[col,b,ih,id,is,iscan] $
							+rg_1[b,ih,id,is,iscan,2,1]*dn_y[col,b,ih,id,is,iscan]^2)) $
							+(rvs[b,is,id,ih,1]/rvs[b,is,id,ih,2])*rad_p1[col,b,id,1] $
							-(rvs[b,is,id,ih,1]-rvs[b,is,id,ih,2])/(rvs[b,is,id,ih,2]*p[b]) $
							*(rad_p3_x[col,b,ih,id,iscan,0]-(1-p[b])*rad_p3_x[col,b,ih,id,iscan,1])
						rad_OBC_ret_2[col,b,ih,id,is,iscan,5] $
							=(rg_1[b,ih,id,is,iscan,0,2] $
							+rg_1[b,ih,id,is,iscan,1,2]*dn_y[col,b,ih,id,is,iscan] $
							+rg_1[b,ih,id,is,iscan,2,2]*dn_y[col,b,ih,id,is,iscan]^2 $
							+rg_1[b,ih,id,is,iscan,3,2]*dn_y[col,b,ih,id,is,iscan]^3) $
							/rvs[b,is,id,ih,2]/p[b]*(rad_BCS[col,b,ih,id,is]/(rg_1[b,ih,id,is,iscan,0,2] $
							+rg_1[b,ih,id,is,iscan,1,2]*dn_y[col,b,ih,id,is,iscan] $
							+rg_1[b,ih,id,is,iscan,2,2]*dn_y[col,b,ih,id,is,iscan]^2 $
							+rg_1[b,ih,id,is,iscan,3,2]*dn_y[col,b,ih,id,is,iscan]^3)) $
							+(rvs[b,is,id,ih,1]/rvs[b,is,id,ih,2])*rad_p1[col,b,id,1] $
							-(rvs[b,is,id,ih,1]-rvs[b,is,id,ih,2])/(rvs[b,is,id,ih,2]*p[b]) $
							*(rad_p3_x[col,b,ih,id,iscan,0]-(1-p[b])*rad_p3_x[col,b,ih,id,iscan,1])

						ARD_obc_2[col,b,ih,id,is,iscan,0:5]=100.*(rad_OBC_ret_2[col,b,ih,id,is,iscan,0:5] $
							/rad_p3_x[col,b,ih,id,iscan,4]-1.)

						rad_OBC_bkg_2[col,b,ih,id,is,iscan]=rvs[b,is,id,ih,1]*rad_p1[col,b,id,1,0] $
							-(rvs[b,is,id,ih,1]-rvs[b,is,id,ih,0])/p[b] $
							*(rad_p3_x[col,b,ih,id,iscan,0]-(1-p[b])*rad_p3_x[col,b,ih,id,iscan,1])

;*****************************************************************************
;  Average the ARD and find the OBC emissivity.
;*****************************************************************************

						IF iscan EQ max_scan/2-1 THEN BEGIN
							ARD_obc[col,b,ih,id,is,1] $
								=sig_mean(ARD_obc_2[col,b,ih,id,is,0:max_scan/2-1,1],3,1)
							ARD_obc[col,b,ih,id,is,2] $
								=sig_mean(ARD_obc_2[col,b,ih,id,is,0:max_scan/2-1,2],3,1)
							ARD_obc[col,b,ih,id,is,4] $
								=sig_mean(ARD_obc_2[col,b,ih,id,is,0:max_scan/2-1,4],3,1)
							ARD_obc[col,b,ih,id,is,5] $
								=sig_mean(ARD_obc_2[col,b,ih,id,is,0:max_scan/2-1,5],3,1)

							rad_OBC_bkg_1[col,b,ih,id,is] $
								=sig_mean(rad_OBC_bkg_2[col,b,ih,id,is,0:max_scan/2-1],3,1)

							IF ih EQ 0 THEN continue

							rad_OBC_bkg[col,b,id] $
								=sig_mean(rad_OBC_bkg_1[col,b,0:1,id,0:subsets_TEB[b]-1],3,1)

							rad_OBC_ret[col,b,id,1] $
								=mean(rad_OBC_ret_2[col,b,0:1,id,0:subsets_TEB[b]-1, $
									0:max_scan/2-1,1])
							rad_OBC_ret[col,b,id,2] $
								=mean(rad_OBC_ret_2[col,b,0:1,id,0:subsets_TEB[b]-1, $
									0:max_scan/2-1,2])
							rad_OBC_ret[col,b,id,4] $
								=mean(rad_OBC_ret_2[col,b,0:1,id,0:subsets_TEB[b]-1, $
									0:max_scan/2-1,4])
							rad_OBC_ret[col,b,id,5] $
								=mean(rad_OBC_ret_2[col,b,0:1,id,0:subsets_TEB[b]-1, $
									0:max_scan/2-1,5])

							emiss_OBC_1[col,b,id,1] $
								=(rad_OBC_ret[col,b,id,1]-rad_OBC_bkg[col,b,id]) $
								/(mean(rad_p3_x[col,b,0:1,id,0:max_scan/2-1,4]) $
								-rad_OBC_bkg[col,b,id])
							emiss_OBC_1[col,b,id,2] $
								=(rad_OBC_ret[col,b,id,2]-rad_OBC_bkg[col,b,id]) $
								/(mean(rad_p3_x[col,b,0:1,id,0:max_scan/2-1,4]) $
								-rad_OBC_bkg[col,b,id])
							emiss_OBC_1[col,b,id,4] $
								=(rad_OBC_ret[col,b,id,4]-rad_OBC_bkg[col,b,id]) $
								/(mean(rad_p3_x[col,b,0:1,id,0:max_scan/2-1,4]) $
								-rad_OBC_bkg[col,b,id])
							emiss_OBC_1[col,b,id,5] $
								=(rad_OBC_ret[col,b,id,5]-rad_OBC_bkg[col,b,id]) $
								/(mean(rad_p3_x[col,b,0:1,id,0:max_scan/2-1,4]) $
								-rad_OBC_bkg[col,b,id])
						ENDIF
					ENDFOR
					max_ARD_obc[col,b,ih,is,1]=max(abs(ARD_obc[col,b,ih,mindet[b]:detectors_TEB[b]-1,is,1]))
					max_ARD_obc[col,b,ih,is,2]=max(abs(ARD_obc[col,b,ih,mindet[b]:detectors_TEB[b]-1,is,2]))
					max_ARD_obc[col,b,ih,is,4]=max(abs(ARD_obc[col,b,ih,mindet[b]:detectors_TEB[b]-1,is,4]))
					max_ARD_obc[col,b,ih,is,5]=max(abs(ARD_obc[col,b,ih,mindet[b]:detectors_TEB[b]-1,is,5]))

					emiss_OBC[col,b,1]=mean(emiss_OBC_1[col,b,0:detectors_TEB[b]-1,1])
					emiss_OBC[col,b,2]=mean(emiss_OBC_1[col,b,0:detectors_TEB[b]-1,2])
					emiss_OBC[col,b,4]=mean(emiss_OBC_1[col,b,0:detectors_TEB[b]-1,4])
					emiss_OBC[col,b,5]=mean(emiss_OBC_1[col,b,0:detectors_TEB[b]-1,5])
				ENDFOR
			ENDFOR
		ENDFOR
	ENDFOR
ENDFOR
print,'Emmissivity of OBC (quad and cubic)',mean(emiss_OBC[0:ncollects-1,0:nbands_TEB-1,1]), $
	mean(emiss_OBC[0:ncollects-1,0:nbands_TEB-1,2]),mean(emiss_OBC[0:ncollects-1,0:nbands_TEB-1,4]),mean(emiss_OBC[0:ncollects-1,0:nbands_TEB-1,5]),

save,/compress,rad_OBC_ret,max_ARD_obc,ARD_obc,emiss_OBC,filename=outfile

;*****************************************************************************
;  Save data on ARD to dat file.
;*****************************************************************************

outfile2=outpath+a2[a3-3]+'.'+a2[a3-2]+'.'+a2[a3-1]+'.ard.txt'

fmt_1='(i2, 5x, i3, 5x, i2, 5x, f, 5x, f, 5x, f, 5x, f, 5x, f, 5x, f, 5x, f, 5x, f)'

openw,lun,outfile2,/get_lun
printf,lun,'Absolute Radiance Difference (ARD) (quadratic fit)'
printf,lun,'Collect     Temp       HAM      I4        I5       M12       M13        M14      M15      M16A      M16B'
FOR ih=0, nham-1 DO BEGIN
	FOR col=0, ncollects-1 DO BEGIN
		printf,lun,format=fmt_1,col,T_obc[ih,col],ih,mean(max_ARD_obc[col,0,ih,*,4]),mean(max_ARD_obc[col,1,ih,*,4]), $
			max_ARD_obc[col,2,ih,0,4],max_ARD_obc[col,3,ih,0,4],max_ARD_obc[col,4,ih,0,4], $
			max_ARD_obc[col,5,ih,0,4],max_ARD_obc[col,6,ih,0,4],max_ARD_obc[col,7,ih,0,4]
	ENDFOR
ENDFOR
free_lun,lun

outfile8=outpath+a2[a3-3]+'.'+a2[a3-2]+'.'+a2[a3-1]+'.temporary.txt'

fmt_1='(d)'

openw,lun,outfile8,/get_lun
FOR b=0, nbands_TEB-1 DO BEGIN
	FOR col=0, ncollects-1 DO BEGIN
		printf,lun,format=fmt_1,mean(max_ARD_obc[col,b,0,0:subsets_TEB[b]-1,1])
	ENDFOR
ENDFOR
free_lun,lun
close,lun,/file,/all

END
