;*****************************************************************************
;				NICST TOOL
;
;  RC-5 Part 2 test for TEB bands. Calculates the OBC radiance.
;
;  Input:
;  1) RVS data.
;  2) Band dependent reflectence of the RTA (p).
;  3) Temperatures of instrument (HAM, RTA, CAV, OBC BB, SH, CPFA, OMM,
;     LWASP, MWASP, BCS, and SVS).
;  4) RSR data.
;
;  Output:
;  1) emiss_OBC[band]
;  
;  v1.0 - 10/16/2008 - Jeff McIntire
;*****************************************************************************

;*****************************************************************************
;  Main program.
;*****************************************************************************

PRO RC5_OBC_emiss, infile

;*****************************************************************************
;  Read in input file and sav TEB file.
;*****************************************************************************

nlines=File_Lines(infile)
ab=''
header=strarr(1)
inpath=strarr(1)		; Input path.
outpath=strarr(1)		; Output path.
rvs_in=strarr(1)		; RVS .sav file.
param_in=strarr(1)		; EDU coeff .sav file.
rsrpath=strarr(1)		; RSR .sav file.
fnumber=strarr(1)
header_size=intarr(1)

openr,lun,infile,/Get_Lun	
readf,lun,header	
readf,lun,inpath
readf,lun,outpath
readf,lun,rvs_in
readf,lun,param_in
readf,lun,rsrpath
readf,lun,fnumber
readf,lun,header_size
Free_Lun,lun

n_collect,inpath,ncollects,type,0
restore,'general.TEB.sav'

;*****************************************************************************
;  Some definitions.
;*****************************************************************************

restore,param_in

max_scan=100					; # of scans.

a2=strsplit(outpath,'/',/Extract)
a3=N_ELEMENTS(a2)

outfile=outpath+'/'+a2[a3-3]+'.'+a2[a3-2]+'.'+a2[a3-1]+'.emiss.sav'
print,outfile

emiss_OBC_4=dblarr(ncollects,nbands_TEB,nham,maxdet,maxsubsets,max_scan)
emiss_OBC_3=dblarr(ncollects,nbands_TEB,nham,maxdet,maxsubsets)
emiss_OBC_2=dblarr(nbands_TEB,nham,maxdet,maxsubsets)
emiss_OBC_1=dblarr(nbands_TEB,nham,maxsubsets)
emiss_OBC=dblarr(nbands_TEB)

;*****************************************************************************
;  Read in BCS coefficients
;*****************************************************************************

outpath1='/'
FOR j=0, a3-2 DO BEGIN
	outpath1=outpath1+a2[j]+'/'
ENDFOR

restore,outpath1+a2[a3-3]+'.'+a2[a3-2]+'.rg.sav'
print,outpath1+a2[a3-3]+'.'+a2[a3-2]+'.rg.sav'

;*****************************************************************************
;  Read in the radiance and rvs
;*****************************************************************************

restore,outpath+a2[a3-3]+'.'+a2[a3-2]+'.'+a2[a3-1]+'.emiss.initial.sav'
print,outpath+a2[a3-3]+'.'+a2[a3-2]+'.'+a2[a3-1]+'.emiss.initial.sav'

;*****************************************************************************
;  Find the collect for which the BCS and OBC temperatures are closest
;  (warm-up only).
;*****************************************************************************

temp=fltarr(ncollects)

temp[*]=T_obc[0,*]-T_bcs[*]
FOR col=0, ncollects-2 DO BEGIN
    IF T_obc[0,col]-T_obc[0,col+1] GT 0 THEN temp[col+1]=100
ENDFOR
x=min(abs(temp[*]))

FOR col=0, ncollects-1 DO BEGIN
    IF x EQ abs(temp[col]) THEN target_col=col
ENDFOR

;*****************************************************************************
;  Calculate the BCS and OBC radiances.
;*****************************************************************************
n_collect,inpath,ncollects,type,0

rad_OBC_ret_2=dblarr(ncollects,nbands_TEB,nham,maxdet,maxsubsets)
rad_OBC_ret=dblarr(ncollects,nbands_TEB,maxdet)
rad_OBC_bkg_1=dblarr(ncollects,nbands_TEB,nham,maxdet)
rad_OBC_bkg=dblarr(ncollects,nbands_TEB,maxdet)
emiss_OBC_1=dblarr(ncollects,nbands_TEB,maxdet)

mindet=[0,0,0,0,1,0,0,0]

FOR b=0, nbands_TEB-1 DO BEGIN
    FOR ih=0, nham-1 DO BEGIN
        FOR is=0, subsets_TEB[b]-1 DO BEGIN
            FOR id=0, detectors_TEB[b]-1 DO BEGIN
                FOR col=0, ncollects-1 DO BEGIN
                    IF col NE target_col THEN continue

                    rad_OBC_ret_2[col,b,ih,id,is] $
                      =(rg_1[b,ih,id,is,0,1,0]+rg_1[b,ih,id,is,1,1,0]*mean(dn_x[col,b,ih,id,is,0:49]) $
                        +rg_1[b,ih,id,is,2,1,0]*mean(dn_x[col,b,ih,id,is,0:49])^2)/0.4444 $
                      +rvs[b,is,id,ih,1]*rad_p1[col,b,id,1]-(rvs[b,is,id,ih,1]-1)/p[b] $
                      *(mean(rad_p3_x[col,b,ih,id,0:49,0])-(1-p[b])*mean(rad_p3_x[col,b,ih,id,0:49,1]))
                    
                    rad_OBC_bkg_1[col,b,ih,id] $
                      =F_rta*mean(rad_p3_x[col,b,ih,id,0:49,1])+F_cav*mean(rad_p3_x[col,b,ih,id,0:49,2]) $
                      +F_sh*mean(rad_p3_x[col,b,ih,id,0:49,3])

;*****************************************************************************
;  Find the OBC emissivity.
;*****************************************************************************

                    IF ih EQ 0 THEN continue
                    
                    rad_OBC_bkg[col,b,id]=mean(rad_OBC_bkg_1[col,b,0:1,id])
                    
                    rad_OBC_ret[col,b,id]=mean(rad_OBC_ret_2[col,b,0:1,id,0:subsets_TEB[b]-1])
                    
                    emiss_OBC_1[col,b,id]=rad_OBC_ret[col,b,id]/mean(rad_p3_x[col,b,0:1,id,0:max_scan/2-1,4])

                    IF ih EQ 1 AND id EQ detectors_TEB[b]-1 AND is EQ subsets_TEB[b]-1 THEN $
                      print,b,col,mean(emiss_OBC_1[col,b,mindet[b]:detectors_TEB[b]-1])
                ENDFOR
            ENDFOR
        ENDFOR
    ENDFOR
ENDFOR

save,/compress,emiss_OBC,filename=outfile

END

