;*****************************************************************************
;				NICST TOOL
;
;  RC-5 Part 2 test for TEB bands. Calculates the OBC radiance, gains, 
;  retieved radiance, and emissivity.
;
;  Input:
;  1) Input file.
;
;  Output:
;  1) emiss_OBC[band]
;  
;  v1.0 - 010/16/2008 - Jeff McIntire
;*****************************************************************************

;*****************************************************************************
;  Main program.
;*****************************************************************************

PRO RC5_OBC_emiss_2, infile

;*****************************************************************************
;  Read in input file and sav TEB file.
;*****************************************************************************

nlines=File_Lines(infile)
ab=''
header=strarr(1)
inpath=strarr(1)		; Input path.
outpath=strarr(1)		; Output path.
rvs_in=strarr(1)		; RVS .sav file.
param_in=strarr(1)		; EDU coeff .sav file.
rsrpath=strarr(1)		; RSR .sav file.
fnumber=strarr(1)
header_size=intarr(1)

openr,lun,infile,/Get_Lun	
readf,lun,header	
readf,lun,inpath
readf,lun,outpath
readf,lun,rvs_in
readf,lun,param_in
readf,lun,rsrpath
readf,lun,fnumber
readf,lun,header_size
Free_Lun,lun

n_collect,inpath,ncollects,type,0
restore,'general.TEB.sav'

;*****************************************************************************
;  Some definitions.
;*****************************************************************************

restore,param_in

max_scan=100					; # of scans.

a2=strsplit(outpath,'/',/Extract)
a3=N_ELEMENTS(a2)

outfile=outpath+'/'+a2[a3-3]+'.'+a2[a3-2]+'.'+a2[a3-1]+'.emiss.2.sav'
print,outfile

;*****************************************************************************
;  Read in BCS coefficients
;*****************************************************************************

outpath1='/'
FOR j=0, a3-2 DO BEGIN
	outpath1=outpath1+a2[j]+'/'
ENDFOR

restore,outpath1+a2[a3-3]+'.'+a2[a3-2]+'.rg.sav'
print,outpath1+a2[a3-3]+'.'+a2[a3-2]+'.rg.sav'

;*****************************************************************************
;  Read in the radiance and rvs
;*****************************************************************************

restore,outpath+a2[a3-3]+'.'+a2[a3-2]+'.'+a2[a3-1]+'.emiss.initial.sav'
print,outpath+a2[a3-3]+'.'+a2[a3-2]+'.'+a2[a3-1]+'.emiss.initial.sav'

;*****************************************************************************
;  Start iteration of the OBC emissivity.
;  Assume the OBC emissivity is 1 initially.
;*****************************************************************************

n_collect,inpath,ncollects,type,0
ham=initial_ham_side(inpath,header_size)

rad_OBC_1_scan=dblarr(ncollects,nbands_TEB,nham,maxdet,maxsubsets,max_scan)
rad_OBC_scan=dblarr(ncollects,nbands_TEB,nham,maxdet,maxsubsets,max_scan)
rad_OBC_scan_x=dblarr(ncollects,nbands_TEB,nham,maxdet,maxsubsets,max_scan/2)

x1=dblarr(ncollects)
x2=dblarr(ncollects)
z1=dblarr(ncollects)
z2=dblarr(ncollects)

rg_1=dblarr(nbands_TEB,nham,maxdet,maxsubsets,max_scan/2,3,4)
dL_fit=dblarr(ncollects,nbands_TEB,nham,maxdet,maxsubsets,max_scan/2,4)

rad_OBC_ret_2=dblarr(ncollects,nbands_TEB,nham,maxdet,maxsubsets,max_scan/2,3)
rad_OBC_ret=dblarr(ncollects,nbands_TEB,maxdet,3)
rad_OBC_bkg_2=dblarr(ncollects,nbands_TEB,nham,maxdet,maxsubsets,max_scan/2)
rad_OBC_bkg_1=dblarr(ncollects,nbands_TEB,nham,maxdet,maxsubsets)
rad_OBC_bkg=dblarr(ncollects,nbands_TEB,maxdet)
emiss_OBC_1=dblarr(ncollects,nbands_TEB,maxdet)
emiss_OBC_x=dblarr(ncollects,nbands_TEB)
emiss_OBC=dblarr(nbands_TEB)

mindet=[0,0,0,0,1,0,0,0]

FOR k=0, 99 DO BEGIN
	
	IF k EQ 0 THEN emiss_OBC[*]=1.0

	FOR b=0, nbands_TEB-1 DO BEGIN
		FOR is=0, subsets_TEB[b]-1 DO BEGIN
			FOR col=0, ncollects-1 DO BEGIN
				IF ham[col] EQ 0 THEN ih=0
				IF ham[col] EQ 1 THEN ih=1
				FOR iscan=0, max_scan-1 DO BEGIN
					rad_OBC_1_scan[col,b,ih,0:detectors_TEB[b]-1,is,iscan]=emiss_OBC[b] $
						*rad_p3[col,b,ih,0:detectors_TEB[b]-1,iscan,4]+(1-emiss_OBC[b]) $
						*(f_cav*rad_p3[col,b,ih,0:detectors_TEB[b]-1,iscan,2] $
						+f_sh*rad_p3[col,b,ih,0:detectors_TEB[b]-1,iscan,3] $
						+f_rta*rad_p3[col,b,ih,0:detectors_TEB[b]-1,iscan,1])

					rad_OBC_scan[col,b,ih,0:detectors_TEB[b]-1,is,iscan] $
						=rad_OBC_1_scan[col,b,ih,0:detectors_TEB[b]-1,is,iscan] $
						-rvs[b,is,0:detectors_TEB[b]-1,ih,1]*rad_p1[col,b,0:detectors_TEB[b]-1,1,0] $
						+(rvs[b,is,0:detectors_TEB[b]-1,ih,1]-1)/p[b] $
						*(rad_p3[col,b,ih,0:detectors_TEB[b]-1,iscan,0] $
						-(1-p[b])*rad_p3[col,b,ih,0:detectors_TEB[b]-1,iscan,1])

					IF ih EQ 0 THEN BEGIN
						ih=1
						goto,skip
					ENDIF
					IF ih EQ 1 THEN ih=0
					skip:
				ENDFOR
			ENDFOR
		ENDFOR
	ENDFOR

	FOR b=0, nbands_TEB-1 DO BEGIN
		FOR is=0, subsets_TEB[b]-1 DO BEGIN
			FOR ih=0, nham-1 DO BEGIN
				FOR id=0, detectors_TEB[b]-1 DO BEGIN
					FOR col=0, ncollects-1 DO BEGIN
						index1=WHERE(abs(rad_OBC_scan[col,b,ih,id,is,*]) GT 0.01, arr2)
						rad_OBC_scan_x[col,b,ih,id,is,0:49]=rad_OBC_scan[col,b,ih,id,is,index1]
					ENDFOR
				ENDFOR
			ENDFOR
		ENDFOR
	ENDFOR

;*****************************************************************************
;  Fitting using POLY_FIT (Quadratic) for both warm-up and cool-down.
;*****************************************************************************

	FOR iscan=0, max_scan/2-1 DO BEGIN
		FOR b=0, nbands_TEB-1 DO BEGIN
			FOR is=0, subsets_TEB[b]-1 DO BEGIN
				FOR ih=0, nham-1 DO BEGIN
					lcol=0
					hcol=ncollects
					FOR col=0, ncollects-2 DO BEGIN
						IF T_obc[ih,col] GT T_obc[ih,col+1] THEN hcol=hcol-1
					ENDFOR

					FOR id=0, detectors_TEB[b]-1 DO BEGIN

						x1=dn_x[lcol:hcol-1,b,ih,id,is,iscan]
						z1=rad_OBC_scan_x[lcol:hcol-1,b,ih,id,is,iscan]*0.4444
						x2=dn_x[hcol:ncollects-1,b,ih,id,is,iscan]
						z2=rad_OBC_scan_x[hcol:ncollects-1,b,ih,id,is,iscan]*0.4444

;*****************************************************************************
;  Conduct quadratic poly_fit to dn vs deltaL (ifit 0) 
;  and deltaL vs dn (ifit 1). These fits are for the warm up.
;*****************************************************************************

						result2=POLY_FIT(x1,z1,2)
						result2_1=POLY_FIT(z1,x1,2)

						rg_1[b,ih,id,is,iscan,0:2,0]=result2[0:2]
						rg_1[b,ih,id,is,iscan,0:2,1]=result2_1[0:2]

;*****************************************************************************
;  Conduct quadratic poly_fit to dn vs deltaL (ifit 2) 
;  and deltaL vs dn (ifit 3). These fits are for the cool down.
;*****************************************************************************

						result5=POLY_FIT(x2,z2,2)
						result5_1=POLY_FIT(z2,x2,2)

						rg_1[b,ih,id,is,iscan,0:2,2]=result5[0:2]
						rg_1[b,ih,id,is,iscan,0:2,3]=result5_1[0:2]

;*****************************************************************************
;  Get the fractional fitting residual.
;*****************************************************************************

						FOR col=lcol, hcol-1 DO BEGIN
							dL_fit[col,b,ih,id,is,iscan,0]= $
								(rad_OBC_scan[col,b,ih,id,is,iscan]*0.4444 $
								-result2[0]-result2[1]*dn_x[col,b,ih,id,is,iscan] $
								-result2[2]*dn_x[col,b,ih,id,is,iscan]^2) $
								/(rad_OBC_scan[col,b,ih,id,is,iscan]*0.4444)
						ENDFOR
						FOR col=hcol-1, ncollects-1 DO BEGIN
							dL_fit[col,b,ih,id,is,iscan,2]= $
								(rad_OBC_scan[col,b,ih,id,is,iscan]*0.4444 $
								-result5[0]-result5[1]*dn_x[col,b,ih,id,is,iscan] $
								-result5[2]*dn_x[col,b,ih,id,is,iscan]^2) $
								/(rad_OBC_scan[col,b,ih,id,is,iscan]*0.4444)
						ENDFOR
					ENDFOR
				ENDFOR
			ENDFOR
		ENDFOR
	ENDFOR

;*****************************************************************************
;  Calculate the retrieved radiance.
;*****************************************************************************

	FOR iscan=0, max_scan/2-1 DO BEGIN
		FOR b=0, nbands_TEB-1 DO BEGIN
			FOR ih=0, nham-1 DO BEGIN
				FOR is=0, subsets_TEB[b]-1 DO BEGIN
					lcol=0
					hcol=ncollects
					FOR col=0, ncollects-2 DO BEGIN
						IF T_obc[ih,col] GT T_obc[ih,col+1] THEN hcol=hcol-1
					ENDFOR

;*****************************************************************************
;  Calculate the retrieved radiance for the warm-up.
;*****************************************************************************

					FOR col=0, ncollects-1 DO BEGIN
						FOR id=0, detectors_TEB[b]-1 DO BEGIN
							rad_OBC_ret_2[col,b,ih,id,is,iscan,1] $
								=(rg_1[b,ih,id,is,iscan,0,0] $
								+rg_1[b,ih,id,is,iscan,1,0]*dn_x[col,b,ih,id,is,iscan] $
								+rg_1[b,ih,id,is,iscan,2,0]*dn_x[col,b,ih,id,is,iscan]^2) $
								/0.4444/rvs[b,is,id,ih,2] $
								+(rvs[b,is,id,ih,1]/rvs[b,is,id,ih,2])*rad_p1[col,b,id,1] $
								-(rvs[b,is,id,ih,1]-rvs[b,is,id,ih,2])/(rvs[b,is,id,ih,2]*p[b]) $
								*(rad_p3_x[col,b,ih,id,iscan,0] $
								-(1-p[b])*rad_p3_x[col,b,ih,id,iscan,1])

							rad_OBC_bkg_2[col,b,ih,id,is,iscan] $
								=rvs[b,is,id,ih,1]*rad_p1[col,b,id,1,0] $
								-(rvs[b,is,id,ih,1]-rvs[b,is,id,ih,0])/p[b] $
								*(rad_p3_x[col,b,ih,id,iscan,0] $
								-(1-p[b])*rad_p3_x[col,b,ih,id,iscan,1])

;*****************************************************************************
;  Find the OBC emissivity.
;*****************************************************************************

							IF iscan EQ max_scan/2-1 THEN BEGIN
								rad_OBC_bkg_1[col,b,ih,id,is] $
									=mean(rad_OBC_bkg_2[col,b,ih,id,is,0:max_scan/2-1])

								IF ih EQ 0 THEN continue

								rad_OBC_bkg[col,b,id] $
									=mean(rad_OBC_bkg_1[col,b,0:1,id,0:subsets_TEB[b]-1])

								rad_OBC_ret[col,b,id,1] $
									=mean(rad_OBC_ret_2[col,b,0:1,id,0:subsets_TEB[b]-1, $
										0:max_scan/2-1,1])

								emiss_OBC_1[col,b,id] $
									=(rad_OBC_ret[col,b,id,1]-rad_OBC_bkg[col,b,id]) $
									/(sig_mean(rad_OBC_1_scan[col,b,0:1,id, $
									0:subsets_TEB[b]-1,0:max_scan-1],3,1) $
									-rad_OBC_bkg[col,b,id])
								emiss_OBC_1[col,b,id] $
									=(rad_OBC_ret[col,b,id,1]-rad_OBC_bkg[col,b,id]) $
									/(sig_mean(rad_p3[col,b,0:1,id,0:max_scan-1,4],3,1) $
									-rad_OBC_bkg[col,b,id])
								emiss_OBC_1[col,b,id] $
									=(rad_OBC_ret[col,b,id,1]) $
									/(sig_mean(rad_p3[col,b,0:1,id,0:max_scan-1,4],3,1))
							ENDIF
						ENDFOR
						emiss_OBC_x[col,b]=mean(emiss_OBC_1[col,b,mindet[b]:detectors_TEB[b]-1])
					ENDFOR
					emiss_OBC[b]=mean(emiss_OBC_x[0:ncollects-1,b])
				ENDFOR
			ENDFOR
		ENDFOR
	ENDFOR
	print,'Emmissivity of OBC',emiss_OBC[*]
ENDFOR

save,/compress,emiss_OBC,filename=outfile

END

