;*****************************************************************************
;				NICST TOOL
;
;  Uses polynominal fits, linear (ifit=0),  quadratic (ifit=1) 
;  and cubic equation (ifit=2), relating dn to radiance. 
;
;  Input:
;  1) Input file.
;
;  Output:
;  1) ncollects
;  2) rg[band,detector,a0 a1 a2 a3,ifit,rsr] -- radiometric gain (ham and subsample averaged) per fit.
;  3) rg_1[band,HAM,detector,subsample,a0 a1 a2 a3,ifit,rsr] -- radiometric gain per fit.
;  4) dL_fit[collect,band,HAM,detector,subsample,ifit,rsr] -- fitting residual per fit.
;  
;  v1.0 - 03/27/2008 - Jeff McIntire
;  v2.0 - 05/27/2008 - Jeff McIntire
;*****************************************************************************

;*****************************************************************************
;  Call the simplex to get the radiometric gains.
;*****************************************************************************

PRO RC5_OBC_fit, infile

;*****************************************************************************
;  Read in input file, sav TEB file, and make some definitions.
;*****************************************************************************

nlines=File_Lines(infile)
ab=''
header=strarr(1)
inpath=strarr(1)		; Input path.
outpath=strarr(1)		; Output path.
rvs_in=strarr(1)
param_in=strarr(1)
rsrpath=strarr(1)
fnumber=fltarr(1)

openr,lun,infile,/Get_Lun	
readf,lun,header	
readf,lun,inpath
readf,lun,outpath
readf,lun,rvs_in
readf,lun,param_in
readf,lun,rsrpath
readf,lun,fnumber
Free_Lun,lun

a2=strsplit(outpath,'/',/Extract)
a3=N_ELEMENTS(a2)

infname1=outpath+a2[a3-3]+'.'+a2[a3-2]+'.'+a2[a3-1]+'.dn.sav'
infname2=outpath+a2[a3-3]+'.'+a2[a3-2]+'.'+a2[a3-1]+'.rad.sav'
infname3=outpath+a2[a3-3]+'.'+a2[a3-2]+'.'+a2[a3-1]+'.temp.sav'
restore,infname1
restore,infname2
restore,infname3

restore,'general.TEB.sav'

outfile=outpath+a2[a3-3]+'.'+a2[a3-2]+'.'+a2[a3-1]+'.rg.sav'

maxscans=100

x1=dblarr(ncollects)
x2=dblarr(ncollects)
z1=dblarr(ncollects)
z2=dblarr(ncollects)

rg_1=dblarr(nbands_TEB,nham,maxdet,maxsubsets,maxscans,4,12)
dL_fit=dblarr(ncollects,nbands_TEB,nham,maxdet,maxsubsets,maxscans,12)

;*****************************************************************************
;  Reform the dn and radiance to get rid of extra zeros.
;*****************************************************************************

dn_x=dblarr(ncollects,nbands_TEB,nham,maxdet,maxsubsets,maxscans)
rad_OBC_scan_x=dblarr(ncollects,nbands_TEB,nham,maxdet,maxsubsets,maxscans)

FOR b=0, nbands_TEB-1 DO BEGIN
	FOR is=0, subsets_TEB[b]-1 DO BEGIN
		FOR ih=0, nham-1 DO BEGIN
			FOR id=0, detectors_TEB[b]-1 DO BEGIN
				FOR col=0, ncollects-1 DO BEGIN
					index1=WHERE(dn_col[col,b,ih,id,is,1,*,2] NE 0, arr1)
					index2=WHERE(abs(rad_OBC_scan[col,b,ih,id,is,*]) GT 0.01, arr2)
					dn_x[col,b,ih,id,is,0:49]=dn_col[col,b,ih,id,is,1,index1,2]
					rad_OBC_scan_x[col,b,ih,id,is,0:49]=rad_OBC_scan[col,b,ih,id,is,index2]
				ENDFOR
			ENDFOR
		ENDFOR
	ENDFOR
ENDFOR

;*****************************************************************************
;  Fitting using POLY_FIT (Linear and Quadratic and Cubic).
;*****************************************************************************

FOR iscan=0, maxscans/2-1 DO BEGIN
	FOR b=0, nbands_TEB-1 DO BEGIN
		FOR is=0, subsets_TEB[b]-1 DO BEGIN
			FOR ih=0, nham-1 DO BEGIN
				lcol=0
				hcol=ncollects
				FOR col=0, ncollects-2 DO BEGIN
					IF T_obc[ih,col] GT T_obc[ih,col+1] THEN hcol=hcol-1
				ENDFOR

				FOR id=0, detectors_TEB[b]-1 DO BEGIN

					x1=dn_x[lcol:hcol-1,b,ih,id,is,iscan]
					z1=rad_OBC_scan_x[lcol:hcol-1,b,ih,id,is,iscan]
					x2=dn_x[hcol:ncollects-1,b,ih,id,is,iscan]
					z2=rad_OBC_scan_x[hcol:ncollects-1,b,ih,id,is,iscan]

;*****************************************************************************
;  Conduct linear, quadratic, and cubic poly_fits to dn vs deltaL (ifit 0-2) 
;  and deltaL vs dn (ifit 3-5). These fits are for the warm up.
;*****************************************************************************

					result1=POLY_FIT(x1,z1,1)
					result2=POLY_FIT(x1,z1,2)
					result3=POLY_FIT(x1,z1,3)
					result1_1=POLY_FIT(z1,x1,1)
					result2_1=POLY_FIT(z1,x1,2)
					result3_1=POLY_FIT(z1,x1,3)

					rg_1[b,ih,id,is,iscan,0:1,0]=result1[0:1]
					rg_1[b,ih,id,is,iscan,0:2,1]=result2[0:2]
					rg_1[b,ih,id,is,iscan,*,2]=result3[*]
					rg_1[b,ih,id,is,iscan,0:1,3]=result1_1[0:1]
					rg_1[b,ih,id,is,iscan,0:2,4]=result2_1[0:2]
					rg_1[b,ih,id,is,iscan,*,5]=result3_1[*]

;*****************************************************************************
;  Conduct linear, quadratic, and cubic poly_fits to dn vs deltaL (ifit 6-8) 
;  and deltaL vs dn (ifit 9-11). These fits are for the cool down.
;*****************************************************************************

					result4=POLY_FIT(x2,z2,1)
					result5=POLY_FIT(x2,z2,2)
					result6=POLY_FIT(x2,z2,3)
					result4_1=POLY_FIT(z2,x2,1)
					result5_1=POLY_FIT(z2,x2,2)
					result6_1=POLY_FIT(z2,x2,3)

					rg_1[b,ih,id,is,iscan,0:1,6]=result4[0:1]
					rg_1[b,ih,id,is,iscan,0:2,7]=result5[0:2]
					rg_1[b,ih,id,is,iscan,*,8]=result6[*]
					rg_1[b,ih,id,is,iscan,0:1,9]=result4_1[0:1]
					rg_1[b,ih,id,is,iscan,0:2,10]=result5_1[0:2]
					rg_1[b,ih,id,is,iscan,*,11]=result6_1[*]

;*****************************************************************************
;  Get the fitting residual, fractional fitting residual, radiometric
;  response characterization uncertainty, and radiometric response non-linearity.
;*****************************************************************************

					FOR col=lcol, hcol-1 DO BEGIN
						dL_fit[col,b,ih,id,is,iscan,0]= $
							(rad_OBC_scan_x[col,b,ih,id,is,iscan]-result1[0]-result1[1] $
							*dn_x[col,b,ih,id,is,iscan]) $
							/(rad_OBC_scan_x[col,b,ih,id,is,iscan])
						dL_fit[col,b,ih,id,is,iscan,1]= $
							(rad_OBC_scan_x[col,b,ih,id,is,iscan]-result2[0]-result2[1] $
							*dn_x[col,b,ih,id,is,iscan]-result2[2] $
							*dn_x[col,b,ih,id,is,iscan]^2) $
							/(rad_OBC_scan_x[col,b,ih,id,is,iscan])
						dL_fit[col,b,ih,id,is,iscan,2]= $
							(rad_OBC_scan_x[col,b,ih,id,is,iscan]-result3[0]-result3[1] $
							*dn_x[col,b,ih,id,is,iscan]-result3[2] $
							*dn_x[col,b,ih,id,is,iscan]^2-result3[3] $
							*dn_x[col,b,ih,id,is,iscan]^3) $
							/(rad_OBC_scan_x[col,b,ih,id,is,iscan])
					ENDFOR
					FOR col=hcol-1, ncollects-1 DO BEGIN
						dL_fit[col,b,ih,id,is,iscan,7]= $
							(rad_OBC_scan_x[col,b,ih,id,is,iscan]-result5[0]-result5[1] $
							*dn_x[col,b,ih,id,is,iscan]-result5[2] $
							*dn_x[col,b,ih,id,is,iscan]^2) $
							/(rad_OBC_scan_x[col,b,ih,id,is,iscan])
						dL_fit[col,b,ih,id,is,iscan,8]= $
							(rad_OBC_scan_x[col,b,ih,id,is,iscan]-result6[0]-result6[1] $
							*dn_x[col,b,ih,id,is,iscan]-result6[2] $
							*dn_x[col,b,ih,id,is,iscan]^2-result6[3] $
							*dn_x[col,b,ih,id,is,iscan]^3) $
							/(rad_OBC_scan_x[col,b,ih,id,is,iscan])
					ENDFOR
				ENDFOR
			ENDFOR
		ENDFOR
	ENDFOR
ENDFOR

rg=dblarr(nbands_TEB,nham,maxdet,maxsubsets,2)
FOR b=0, nbands_TEB-1 DO BEGIN
	FOR ih=0, nham-1 DO BEGIN
		FOR is=0, subsets_TEB[b]-1 DO BEGIN
			FOR id=0, detectors_TEB[b]-1 DO BEGIN
				rg[b,ih,id,is,0]=mean(rg_1[b,ih,id,is,0:maxscans/2-1,1,4])
				rg[b,ih,id,is,1]=mean(rg_1[b,ih,id,is,0:maxscans/2-1,1,10])
			ENDFOR
			print,'Warm-up   ',band_name_TEB[b],ih,is,mean(rg[b,ih,1:detectors_TEB[b]-1,is,0]), $
				stddev(rg[b,ih,1:detectors_TEB[b]-1,is,0])
			print,'Cool-down ',band_name_TEB[b],ih,is,mean(rg[b,ih,1:detectors_TEB[b]-1,is,1]), $
				stddev(rg[b,ih,1:detectors_TEB[b]-1,is,1])
		ENDFOR
	ENDFOR
ENDFOR

save,/compress,ncollects,rg_1,dL_fit,rad_OBC_scan_x,dn_x,filename=outfile

;*****************************************************************************
;  Ouput gains to txt files.
;*****************************************************************************

outfile2=outpath+a2[a3-3]+'.'+a2[a3-2]+'.'+a2[a3-1]+'.rg.txt'
outfile3=outpath+a2[a3-3]+'.'+a2[a3-2]+'.'+a2[a3-1]+'.rg.scan_avg.txt'
outfile4=outpath+a2[a3-3]+'.'+a2[a3-2]+'.'+a2[a3-1]+'.rg.det_avg.txt'

fmt_1='(a5, 5x, i2, 5x, i2, 5x, i2, 5x, i2, 5x, d, 5x, d)'
fmt_2='(a5, 5x, i2, 5x, i2, 5x, d, 5x, d)'
fmt_3='(a5, 5x, i2, 5x, d, 5x, d)'

openw,lun,outfile2,/get_lun
printf,lun,'Radiametric gains'
printf,lun,'Band   HAM   Detector   Subset   Gain(warm-up)   Gain(cool-down)'
FOR b=0, nbands_TEB-1 DO BEGIN
	FOR ih=0, nham-1 DO BEGIN
		FOR idet=0, detectors_TEB[b]-1 DO BEGIN
			FOR isubset=0, subsets_TEB[b]-1 DO BEGIN
				FOR iscan=0, maxscans/2-1 DO BEGIN
					printf,lun,format=fmt_1,band_name_TEB[b],ih,idet,isubset,iscan, $
						rg_1[b,ih,idet,is,iscan,1,4],rg_1[b,ih,idet,is,iscan,1,10]
				ENDFOR
			ENDFOR
		ENDFOR
	ENDFOR
ENDFOR
free_lun,lun

openw,lun,outfile3,/get_lun
printf,lun,'Radiametric gains (subset and scan averaged)'
printf,lun,'Band   HAM   Detector   Gain(warm-up)   Gain(cool-down)'
FOR b=0, nbands_TEB-1 DO BEGIN
	FOR ih=0, nham-1 DO BEGIN
		FOR idet=0, detectors_TEB[b]-1 DO BEGIN
			printf,lun,format=fmt_2,band_name_TEB[b],ih,idet,$
				mean(rg_1[b,ih,idet,0:subsets_TEB[b]-1,0:maxscans/2-1,1,4]), $
				mean(rg_1[b,ih,idet,0:subsets_TEB[b]-1,0:maxscans/2-1,1,10])
		ENDFOR
	ENDFOR
ENDFOR
free_lun,lun

openw,lun,outfile4,/get_lun
printf,lun,'Radiametric gains (detector, subset, and scan averaged)'
printf,lun,'Band   HAM   Gain(warm-up)   Gain(cool-down)'
FOR b=0, nbands_TEB-1 DO BEGIN
	FOR ih=0, nham-1 DO BEGIN
		printf,lun,format=fmt_3,band_name_TEB[b],ih, $
			mean(rg_1[b,ih,0:detectors_TEB[b]-1,0:subsets_TEB[b]-1,0:maxscans/2-1,1,4]), $
			mean(rg_1[b,ih,0:detectors_TEB[b]-1,0:subsets_TEB[b]-1,0:maxscans/2-1,1,10])
	ENDFOR
ENDFOR

outfile8=outpath+a2[a3-3]+'.'+a2[a3-2]+'.'+a2[a3-1]+'.residual.warm-up.txt'

fmt_1='(a5, 5x, i2, 5x, i2, 5x, i2, 5x, i2, 5x, d, 5x, d, 5x, d, 5x, d)'

openw,lun,outfile8,/get_lun
printf,lun,'BAND   COLLECT   HAM   SUBSET   DETECTOR   dn   dL_OBC   RESIDUAL(FRACTIONAL)   RESIDUAL'
FOR b=0, nbands_TEB-1 DO BEGIN
	FOR col=0, ncollects-1 DO BEGIN
		IF col GT 7 THEN continue
		FOR ih=0, nham-1 DO BEGIN
			FOR is=0, subsets_TEB[b]-1 DO BEGIN
				FOR id=0, detectors_TEB[b]-1 DO BEGIN
					printf,lun,format=fmt_1,band_name_TEB[b],col,ih,is,id+1, $
						mean(dn_x[col,b,ih,id,is,0:maxscans/2-1]), $
						mean(rad_OBC_scan_x[col,b,ih,id,is,0:maxscans/2-1]), $
						mean(dL_fit[col,b,ih,id,is,0:maxscans/2-1,1]), $
						mean(dL_fit[col,b,ih,id,is,0:maxscans/2-1,1] $
						*rad_OBC_scan_x[col,b,ih,id,is,0:maxscans/2-1])
				ENDFOR
			ENDFOR
		ENDFOR
	ENDFOR
ENDFOR
outfile8=outpath+a2[a3-3]+'.'+a2[a3-2]+'.'+a2[a3-1]+'.residual.cool-down.txt'

fmt_1='(a5, 5x, i2, 5x, i2, 5x, i2, 5x, i2, 5x, d, 5x, d, 5x, d, 5x, d)'

openw,lun,outfile8,/get_lun
printf,lun,'BAND   COLLECT   HAM   SUBSET   DETECTOR   dn   dL_OBC   RESIDUAL(FRACTIONAL)   RESIDUAL'
FOR b=0, nbands_TEB-1 DO BEGIN
	FOR col=0, ncollects-1 DO BEGIN
		IF col LE 7 THEN continue
		FOR ih=0, nham-1 DO BEGIN
			FOR is=0, subsets_TEB[b]-1 DO BEGIN
				FOR id=0, detectors_TEB[b]-1 DO BEGIN
					printf,lun,format=fmt_1,band_name_TEB[b],col,ih,is,id+1, $
						mean(dn_x[col,b,ih,id,is,0:maxscans/2-1]), $
						mean(rad_OBC_scan_x[col,b,ih,id,is,0:maxscans/2-1]), $
						mean(dL_fit[col,b,ih,id,is,0:maxscans/2-1,7]), $
						mean(dL_fit[col,b,ih,id,is,0:maxscans/2-1,7] $
						*rad_OBC_scan_x[col,b,ih,id,is,0:maxscans/2-1])
				ENDFOR
			ENDFOR
		ENDFOR
	ENDFOR
ENDFOR

outfile8=outpath+a2[a3-3]+'.'+a2[a3-2]+'.'+a2[a3-1]+'.coeff.warm-up.txt'

fmt_1='(a5, 5x, i2, 5x, i2, 5x, i2, 5x, d, 5x, d, 5x, d)'

openw,lun,outfile8,/get_lun
printf,lun,'BAND   HAM   SUBSET   DETECTOR   a0   a1   a2'
FOR b=0, nbands_TEB-1 DO BEGIN
	FOR ih=0, nham-1 DO BEGIN
		FOR is=0, subsets_TEB[b]-1 DO BEGIN
			FOR id=0, detectors_TEB[b]-1 DO BEGIN
				printf,lun,format=fmt_1,band_name_TEB[b],ih,is,id+1, $
					mean(rg_1[b,ih,id,is,0:maxscans/2-1,0,1]), $
					mean(rg_1[b,ih,id,is,0:maxscans/2-1,1,1]), $
					mean(rg_1[b,ih,id,is,0:maxscans/2-1,2,1])
			ENDFOR
		ENDFOR
	ENDFOR
ENDFOR
outfile8=outpath+a2[a3-3]+'.'+a2[a3-2]+'.'+a2[a3-1]+'.coeff.cool-down.txt'

fmt_1='(a5, 5x, i2, 5x, i2, 5x, i2, 5x, d, 5x, d, 5x, d)'

openw,lun,outfile8,/get_lun
printf,lun,'BAND   HAM   SUBSET   DETECTOR   a0   a1   a2'
FOR b=0, nbands_TEB-1 DO BEGIN
	FOR ih=0, nham-1 DO BEGIN
		FOR is=0, subsets_TEB[b]-1 DO BEGIN
			FOR id=0, detectors_TEB[b]-1 DO BEGIN
				printf,lun,format=fmt_1,band_name_TEB[b],ih,is,id+1, $
					mean(rg_1[b,ih,id,is,0:maxscans/2-1,0,7]), $
					mean(rg_1[b,ih,id,is,0:maxscans/2-1,1,7]), $
					mean(rg_1[b,ih,id,is,0:maxscans/2-1,2,7])
			ENDFOR
		ENDFOR
	ENDFOR
ENDFOR
free_lun,lun
close,lun,/file,/all

END
