;*****************************************************************************
;  RC5 Part 2 graphs the delta OBC radiance versus dn.
;
;  Output:
;  1) postscript file containing graphs.
;  
;  v1.0 - 04/02/2008 - Jeff McIntire
;*****************************************************************************

PRO RC5_OBC_view, infile

close,/all

restore,'general.TEB.sav'

;*****************************************************************************
;  Read in data.
;*****************************************************************************

nlines=File_Lines(infile)
ab=''
header=strarr(1)
inpath=strarr(1)		; Input path.
outpath=strarr(1)		; Output path.
rvs_in=strarr(1)
param_in=strarr(1)
rsrpath=strarr(1)
fnumber=fltarr(1)

openr,lun,infile,/Get_Lun	
readf,lun,header	
readf,lun,inpath
readf,lun,outpath
readf,lun,rvs_in
readf,lun,param_in
readf,lun,rsrpath
readf,lun,fnumber
Free_Lun,lun

a2=strsplit(outpath,'/',/Extract)
a3=N_ELEMENTS(a2)

set_multicolor,xsave,ysave,psave,chname,lcol,symbol,red,green,blue
set_plot,'ps'
outfname=outpath+a2[a3-3]+'.'+a2[a3-2]+'.'+a2[a3-1]+'.ps'
device,/landscape,filename=outfname,/color
!p.multi=[0,2,1]

infname=strarr(4)
infname[0]=outpath+a2[a3-3]+'.'+a2[a3-2]+'.'+a2[a3-1]+'.rad.sav'
infname[1]=outpath+a2[a3-3]+'.'+a2[a3-2]+'.'+a2[a3-1]+'.dn.sav'
infname[2]=outpath+a2[a3-3]+'.'+a2[a3-2]+'.'+a2[a3-1]+'.rg.sav'
infname[3]=outpath+a2[a3-3]+'.'+a2[a3-2]+'.'+a2[a3-1]+'.temp.sav'
check=FILE_SEARCH(infname[0])
IF check EQ '' THEN BEGIN
	print,'No file is found.',infname[0]
	goto,skip_loop
ENDIF
FOR i=0, 3 DO BEGIN
	print,'Processing file:',infname[i]
	restore,infname[i]
ENDFOR

D=findgen(4000)
E=findgen(30000)
F=fltarr(30000)

;*****************************************************************************
;  Scan average data.
;*****************************************************************************

maxscan=100

rg=fltarr(nbands_TEB,nham,maxdet,maxsubsets,4,12)
dn_1=fltarr(ncollects,nbands_TEB,nham,maxdet,maxsubsets)
dL_fit_1=fltarr(ncollects,nbands_TEB,nham,maxdet,maxsubsets,9)
rad_OBC_1=fltarr(ncollects,nbands_TEB,nham,maxdet,maxsubsets)

FOR b=0, nbands_TEB-1 DO BEGIN
	FOR is=0, subsets_TEB[b]-1 DO BEGIN
		FOR ih=0, nham-1 DO BEGIN
			FOR id=0, detectors_TEB[b]-1 DO BEGIN
				FOR i=0, 3 DO BEGIN
					rg[b,ih,id,is,i,3]=mean(rg_1[b,ih,id,is,0:maxscan/2-1,i,3])
					rg[b,ih,id,is,i,4]=mean(rg_1[b,ih,id,is,0:maxscan/2-1,i,4])
					rg[b,ih,id,is,i,5]=mean(rg_1[b,ih,id,is,0:maxscan/2-1,i,5])
					rg[b,ih,id,is,i,10]=mean(rg_1[b,ih,id,is,0:maxscan/2-1,i,10])
					rg[b,ih,id,is,i,11]=mean(rg_1[b,ih,id,is,0:maxscan/2-1,i,11])
				ENDFOR
				FOR col=0, ncollects-1 DO BEGIN
					rad_OBC_1[col,b,ih,id,is]=mean(rad_OBC_scan_x[col,b,ih,id,is,0:maxscan/2-1])
					dn_1[col,b,ih,id,is]=mean(dn_x[col,b,ih,id,is,0:maxscan/2-1])
					dL_fit_1[col,b,ih,id,is,0]=mean(dL_fit[col,b,ih,id,is,0:maxscan/2-1,0])
					dL_fit_1[col,b,ih,id,is,1]=mean(dL_fit[col,b,ih,id,is,0:maxscan/2-1,1])
					dL_fit_1[col,b,ih,id,is,2]=mean(dL_fit[col,b,ih,id,is,0:maxscan/2-1,2])
					dL_fit_1[col,b,ih,id,is,7]=mean(dL_fit[col,b,ih,id,is,0:maxscan/2-1,7])
					dL_fit_1[col,b,ih,id,is,8]=mean(dL_fit[col,b,ih,id,is,0:maxscan/2-1,8])
				ENDFOR
			ENDFOR
		ENDFOR
	ENDFOR
ENDFOR

;*****************************************************************************
;  Plot gains versus detector.
;*****************************************************************************

!p.multi=[0,3,3]
!p.position=0

FOR b=0, nbands_TEB-1 DO BEGIN

	title1='Detector Variation in Gain (averaged over scans) for Warm-up'
	title2=band_name_TEB[b]+', HAM=1 (quadratic poly_fit)'

	ymax=max(rg[b,0,0:detectors_TEB[b]-1,0,1,4],min=ymin)

	plot,D[0:detectors_TEB[b]-1]+1,rg[b,0,0:detectors_TEB[b]-1,0,1,4],yrange=[ymin,ymax],xrange=[0,detectors_TEB[b]], $
		xtit='Detector',ytit='Gain',title=title2,charsize=1.2,charthick=5,/nodata
	oplot,D[0:detectors_TEB[b]-1]+1,rg[b,0,0:detectors_TEB[b]-1,0,1,4],symsize=1,color=lcol[b],psym=symbol[b]
	oplot,D[0:detectors_TEB[b]-1]+1,rg[b,0,0:detectors_TEB[b]-1,0,1,4],color=lcol[b]
ENDFOR
XYOUTS,0.5,1.025,title1,/normal,alignment=0.5,charsize=1.6,charthick=5

!p.multi=[0,3,3]
!p.position=0

FOR b=0, nbands_TEB-1 DO BEGIN

	title1='Detector Variation in Gain (averaged over scans) for Cool-down'
	title2=band_name_TEB[b]+', HAM=1 (quadratic poly_fit)'

	ymax=max(rg[b,0,0:detectors_TEB[b]-1,0,1,10],min=ymin)

	plot,D[0:detectors_TEB[b]-1]+1,rg[b,0,0:detectors_TEB[b]-1,0,1,10],yrange=[ymin,ymax],xrange=[0,detectors_TEB[b]], $
		xtit='Detector',ytit='Gain',title=title2,charsize=1.2,charthick=5,/nodata
	oplot,D[0:detectors_TEB[b]-1]+1,rg[b,0,0:detectors_TEB[b]-1,0,1,10],symsize=1,color=lcol[b],psym=symbol[b]
	oplot,D[0:detectors_TEB[b]-1]+1,rg[b,0,0:detectors_TEB[b]-1,0,1,10],color=lcol[b]
ENDFOR
XYOUTS,0.5,1.025,title1,/normal,alignment=0.5,charsize=1.6,charthick=5

!p.multi=[0,3,3]
!p.position=0

name=['Gain (OBC warm-up)','Gain (OBC cool-down)']
FOR b=0, nbands_TEB-1 DO BEGIN

	title1='Comparison of Warm-up and Cool-down Gain (averaged over scans) per Detector'
	title2=band_name_TEB[b]+', HAM=1 (quadratic poly_fit)'

	ymax1=max(rg[b,0,0:detectors_TEB[b]-1,0,1,4],min=ymin1)
	ymax2=max(rg[b,0,0:detectors_TEB[b]-1,0,1,10],min=ymin2)
	IF ymax1 GT ymax2 THEN ymax=ymax1 ELSE ymax=ymax2
	IF ymin1 LT ymin2 THEN ymin=ymin1 ELSE ymin=ymin2

	plot,D[0:detectors_TEB[b]-1]+1,rg[b,0,0:detectors_TEB[b]-1,0,1,10],yrange=[ymin,ymax],xrange=[0,detectors_TEB[b]], $
		xtit='Detector',ytit='Gain',title=title2,charsize=1.2,charthick=5,/nodata
	oplot,D[0:detectors_TEB[b]-1]+1,rg[b,0,0:detectors_TEB[b]-1,0,1,4],symsize=1,color=lcol[0],psym=symbol[0]
	oplot,D[0:detectors_TEB[b]-1]+1,rg[b,0,0:detectors_TEB[b]-1,0,1,4],color=lcol[0]
	oplot,D[0:detectors_TEB[b]-1]+1,rg[b,0,0:detectors_TEB[b]-1,0,1,10],symsize=1,color=lcol[1],psym=symbol[1]
	oplot,D[0:detectors_TEB[b]-1]+1,rg[b,0,0:detectors_TEB[b]-1,0,1,10],color=lcol[1]
ENDFOR
XYOUTS,0.5,1.025,title1,/normal,alignment=0.5,charsize=1.6,charthick=5
legend,name,linestyle=[0,0],colors=[0,1],thick=2,charthick=5,pos=[0.7,0.3],/normal

;*****************************************************************************
;  Determine the cutoff between warm-up and cool-down.
;*****************************************************************************

hcol=ncollects
FOR col=0, ncollects-2 DO BEGIN
	IF T_obc[0,col] GT T_obc[0,col+1] THEN hcol=hcol-1
ENDFOR

;*****************************************************************************
;  Plot delta OBC radiance versus dn with fit (quadratic pol_fit).
;*****************************************************************************

!p.multi=[0,3,3]
!p.position=0

title1='dn_obc vs dL_OBC (warm-up)'

FOR b=0, nbands_TEB-1 DO BEGIN

	IF band_type_TEB[b] EQ 0 THEN title2=band_name_TEB[b]+', HAM=1 (quadratic poly_fit)'
	IF band_type_TEB[b] EQ 1 THEN title2=band_name_TEB[b]+' SS1, HAM=1 (quadratic poly_fit)'

	xmax=max(rad_OBC_1[0:hcol-1,b,0,*,0])
	ymax=max(dn_1[0:hcol-1,b,0,*,0])

	plot,rad_OBC_1[0:hcol-1,b,0,1,0],dn_1[0:hcol-1,b,0,1,0],yrange=[0,ymax],xrange=[0,xmax], $
		xtit='dL_OBC',ytit='dn_obc',title=title2,charsize=1.2,charthick=5,/nodata

	FOR id=0, detectors_TEB[b]-1 DO BEGIN
		oplot,rad_OBC_1[0:hcol-1,b,0,id,0],dn_1[0:hcol-1,b,0,id,0],symsize=1,color=lcol[id],psym=symbol[id]
		oplot,rg[b,0,id,0,0,4]+rg[b,0,id,0,1,4]*D+rg[b,0,id,0,2,4]*D^2,color=lcol[id]
	ENDFOR
	oplot,[L_max_TEB[b,0],L_max_TEB[b,0]],[-100,10000],symsize=1,color=lcol[1],linestyle=2,thick=2
	oplot,[L_typ_TEB[b,0],L_typ_TEB[b,0]],[-100,10000],symsize=1,color=lcol[2],linestyle=2,thick=2
	oplot,[L_min_TEB[b,0],L_min_TEB[b,0]],[-100,10000],symsize=1,color=lcol[3],linestyle=2,thick=2

	XYOUTS,0.5,1.025,title1,/normal,alignment=0.5,charsize=1.6,charthick=5
ENDFOR

!p.multi=[0,3,3]
!p.position=0

title1='dn_obc vs dL_OBC (cool-down)'

FOR b=0, nbands_TEB-1 DO BEGIN

	IF band_type_TEB[b] EQ 0 THEN title2=band_name_TEB[b]+', HAM=1 (quadratic poly_fit)'
	IF band_type_TEB[b] EQ 1 THEN title2=band_name_TEB[b]+' SS1, HAM=1 (quadratic poly_fit)'

	xmax=max(rad_OBC_1[hcol:ncollects-1,b,0,*,0])
	ymax=max(dn_1[hcol:ncollects-1,b,0,*,0])

	plot,rad_OBC_1[hcol:ncollects-1,b,0,1,0],dn_1[hcol:ncollects-1,b,0,1,0],yrange=[0,ymax],xrange=[0,xmax], $
		xtit='dL_OBC',ytit='dn_obc',title=title2,charsize=1.2,charthick=5,/nodata

	FOR id=0, detectors_TEB[b]-1 DO BEGIN
		oplot,rad_OBC_1[hcol:ncollects-1,b,0,id,0],dn_1[hcol:ncollects-1,b,0,id,0],symsize=1, $
			color=lcol[id],psym=symbol[id]
		oplot,rg[b,0,id,0,0,4]+rg[b,0,id,0,1,4]*D+rg[b,0,id,0,2,4]*D^2,color=lcol[id]
	ENDFOR
	oplot,[L_max_TEB[b,0],L_max_TEB[b,0]],[-100,10000],symsize=1,color=lcol[1],linestyle=2,thick=2
	oplot,[L_typ_TEB[b,0],L_typ_TEB[b,0]],[-100,10000],symsize=1,color=lcol[2],linestyle=2,thick=2
	oplot,[L_min_TEB[b,0],L_min_TEB[b,0]],[-100,10000],symsize=1,color=lcol[3],linestyle=2,thick=2

	XYOUTS,0.5,1.025,title1,/normal,alignment=0.5,charsize=1.6,charthick=5
ENDFOR

;*****************************************************************************
;  Plot delta OBC radiance versus dn with fits (linear poly_fit, quadratic pol_fit,
;  and cubic poly_fit).
;*****************************************************************************

title=strarr(4)

!p.multi=[0,2,3]

FOR b=0, nbands_TEB-1 DO BEGIN
	!p.position=0

	FOR is=0, subsets_TEB[b]-1 DO BEGIN
		FOR ih=0, nham-1 DO BEGIN
			IF band_type_TEB[b] EQ 0 THEN BEGIN
				title1=band_name_TEB[b]+': dn_obc vs dL_OBC'
				title[0]=band_name_TEB[b]+', '+'HAM='+strtrim(string(ih+1),2)+' (linear poly_fit)'
				title[1]=band_name_TEB[b]+', '+'HAM='+strtrim(string(ih+1),2)+' (quadratic poly_fit)'
				title[2]=band_name_TEB[b]+', '+'HAM='+strtrim(string(ih+1),2)+' (cubic poly_fit)'
			ENDIF
			IF band_type_TEB[b] EQ 1 THEN BEGIN
				title1=band_name_TEB[b]+' SS'+strtrim(string(is+1),2)+': dn_obc vs dL_OBC'
				title[0]=band_name_TEB[b]+' SS'+strtrim(string(is+1),2)+', '+'HAM='+strtrim(string(ih+1),2) $
					+' (linear poly_fit)'
				title[1]=band_name_TEB[b]+' SS'+strtrim(string(is+1),2)+', '+'HAM='+strtrim(string(ih+1),2) $
					+' (quadratic poly_fit)'
				title[2]=band_name_TEB[b]+' SS'+strtrim(string(is+1),2)+', '+'HAM='+strtrim(string(ih+1),2) $
					+' (cubic poly_fit)'
			ENDIF

			xmax=max(L_max_TEB[b,0])
			ymax=max(dn_1[0:hcol-1,b,ih,*,is])

			FOR j=0, 2 DO BEGIN
				plot,rad_OBC_1[0:hcol-1,b,ih,1,is],dn_1[0:hcol-1,b,ih,1,is],yrange=[0,ymax],xrange=[0,xmax], $
					xtit='dL_OBC',ytit='dn_obc',title=title[j],charsize=1.2,charthick=5,/nodata

				FOR id=0, detectors_TEB[b]-1 DO BEGIN
					oplot,rad_OBC_1[0:hcol-1,b,ih,id,is],dn_1[0:hcol-1,b,ih,id,is],symsize=1, $
						color=lcol[id],psym=symbol[id]
					IF j EQ 0 THEN oplot,rg[b,ih,id,is,0,3]+rg[b,ih,id,is,1,3]*D,color=lcol[id]
					IF j EQ 1 THEN oplot,rg[b,ih,id,is,0,4]+rg[b,ih,id,is,1,4]*D $
						+rg[b,ih,id,is,2,4]*D^2,color=lcol[id]
					IF j EQ 2 THEN oplot,rg[b,ih,id,is,0,5]+rg[b,ih,id,is,1,5]*D $
						+rg[b,ih,id,is,2,5]*D^2+rg[b,ih,id,is,3,5]*D^3,color=lcol[id]
				ENDFOR
				oplot,[L_max_TEB[b,0],L_max_TEB[b,0]],[-100,10000], $
					symsize=1,color=lcol[1],linestyle=2,thick=2
				oplot,[L_typ_TEB[b,0],L_typ_TEB[b,0]],[-100,10000], $
					symsize=1,color=lcol[2],linestyle=2,thick=2
				oplot,[L_min_TEB[b,0],L_min_TEB[b,0]],[-100,10000], $
					symsize=1,color=lcol[3],linestyle=2,thick=2

				plot,rad_OBC_1[0:hcol-1,b,ih,0,is],dL_fit_1[0:hcol-1,b,ih,0,is,j], $
					xtit='dL_OBC',ytit='Fitting Residual', $
					xrange=[0,xmax],yrange=[-0.1,0.1],title=title[j],charsize=1.2,charthick=5,/nodata

				FOR id=0, detectors_TEB[b]-1 DO BEGIN
					oplot,rad_OBC_1[0:hcol-1,b,ih,id,is],dL_fit_1[0:hcol-1,b,ih,id,is,j],symsize=1, $
						color=lcol[id],psym=symbol[id]
					oplot,rad_OBC_1[0:hcol-1,b,ih,id,is],dL_fit_1[0:hcol-1,b,ih,id,is,j],color=lcol[id]
				ENDFOR
				oplot,[L_max_TEB[b,0],L_max_TEB[b,0]],[-100,10000], $
					symsize=1,color=lcol[1],linestyle=2,thick=2
				oplot,[L_typ_TEB[b,0],L_typ_TEB[b,0]],[-100,10000], $
					symsize=1,color=lcol[2],linestyle=2,thick=2
				oplot,[L_min_TEB[b,0],L_min_TEB[b,0]],[-100,10000], $
					symsize=1,color=lcol[3],linestyle=2,thick=2
			ENDFOR
			XYOUTS,0.5,1.025,title1,/normal,alignment=0.5,charsize=1.6,charthick=5
			IF band_type_TEB[b] EQ 0 THEN set_legend,1,chname,lcol,symbol
			IF band_type_TEB[b] EQ 1 THEN set_legend,2,chname,lcol,symbol
		ENDFOR
	ENDFOR
ENDFOR

!p.multi=[0,2,2]

FOR b=0, nbands_TEB-1 DO BEGIN
	!p.position=0

	FOR is=0, subsets_TEB[b]-1 DO BEGIN
		FOR ih=0, nham-1 DO BEGIN
			IF band_type_TEB[b] EQ 0 THEN BEGIN
				title1=band_name_TEB[b]+': dn_obc vs dL_OBC (cool-down)'
				title[1]=band_name_TEB[b]+', '+'HAM='+strtrim(string(ih+1),2)+' (quadratic poly_fit)'
				title[2]=band_name_TEB[b]+', '+'HAM='+strtrim(string(ih+1),2)+' (cubic poly_fit)'
			ENDIF
			IF band_type_TEB[b] EQ 1 THEN BEGIN
				title1=band_name_TEB[b]+' SS'+strtrim(string(is+1),2)+': dn_obc vs dL_OBC (cool-down)'
				title[1]=band_name_TEB[b]+' SS'+strtrim(string(is+1),2)+', '+'HAM='+strtrim(string(ih+1),2) $
					+' (quadratic poly_fit)'
				title[2]=band_name_TEB[b]+' SS'+strtrim(string(is+1),2)+', '+'HAM='+strtrim(string(ih+1),2) $
					+' (cubic poly_fit)'
			ENDIF

			xmax=max(L_max_TEB[b,0])
			ymax=max(dn_1[hcol:ncollects-1,b,ih,*,is])

			FOR j=1, 2 DO BEGIN
				plot,rad_OBC_1[hcol:ncollects-1,b,ih,1,is],dn_1[hcol:ncollects-1,b,ih,1,is], $
					yrange=[0,ymax],xrange=[0,xmax], $
					xtit='dL_OBC',ytit='dn_obc',title=title[j],charsize=1.2,charthick=5,/nodata

				FOR id=0, detectors_TEB[b]-1 DO BEGIN
					oplot,rad_OBC_1[hcol:ncollects-1,b,ih,id,is],dn_1[hcol:ncollects-1,b,ih,id,is], $
						symsize=1,color=lcol[id],psym=symbol[id]
					IF j EQ 0 THEN oplot,rg[b,ih,id,is,0,3]+rg[b,ih,id,is,1,3]*D,color=lcol[id]
					IF j EQ 1 THEN oplot,rg[b,ih,id,is,0,4]+rg[b,ih,id,is,1,4]*D $
						+rg[b,ih,id,is,2,4]*D^2,color=lcol[id]
					IF j EQ 2 THEN oplot,rg[b,ih,id,is,0,5]+rg[b,ih,id,is,1,5]*D $
						+rg[b,ih,id,is,2,5]*D^2+rg[b,ih,id,is,3,5]*D^3,color=lcol[id]
				ENDFOR
				oplot,[L_max_TEB[b,0],L_max_TEB[b,0]],[-100,10000], $
					symsize=1,color=lcol[1],linestyle=2,thick=2
				oplot,[L_typ_TEB[b,0],L_typ_TEB[b,0]],[-100,10000], $
					symsize=1,color=lcol[2],linestyle=2,thick=2
				oplot,[L_min_TEB[b,0],L_min_TEB[b,0]],[-100,10000], $
					symsize=1,color=lcol[3],linestyle=2,thick=2

				plot,rad_OBC_1[hcol:ncollects-1,b,ih,0,is],dL_fit_1[hcol:ncollects-1,b,ih,0,is,j], $
					xtit='dL_OBC',ytit='Fitting Residual', $
					xrange=[0,xmax],yrange=[-0.1,0.1],title=title[j],charsize=1.2,charthick=5,/nodata

				FOR id=0, detectors_TEB[b]-1 DO BEGIN
					oplot,rad_OBC_1[hcol:ncollects-1,b,ih,id,is],dL_fit_1[hcol:ncollects-1,b,ih,id,is,j], $
						symsize=1,color=lcol[id],psym=symbol[id]
					oplot,rad_OBC_1[hcol:ncollects-1,b,ih,id,is],dL_fit_1[hcol:ncollects-1,b,ih,id,is,j], $
						color=lcol[id]
				ENDFOR
				oplot,[L_max_TEB[b,0],L_max_TEB[b,0]],[-100,10000], $
					symsize=1,color=lcol[1],linestyle=2,thick=2
				oplot,[L_typ_TEB[b,0],L_typ_TEB[b,0]],[-100,10000], $
					symsize=1,color=lcol[2],linestyle=2,thick=2
				oplot,[L_min_TEB[b,0],L_min_TEB[b,0]],[-100,10000], $
					symsize=1,color=lcol[3],linestyle=2,thick=2
			ENDFOR
			XYOUTS,0.5,1.025,title1,/normal,alignment=0.5,charsize=1.6,charthick=5
			IF band_type_TEB[b] EQ 0 THEN set_legend,1,chname,lcol,symbol
			IF band_type_TEB[b] EQ 1 THEN set_legend,2,chname,lcol,symbol
		ENDFOR
	ENDFOR
ENDFOR

device,/close

skip_loop:

END
