;*****************************************************************************
;				NICST TOOL
;
;  RC5 Part 2 graphs of the specifications.
;  
;  Output:
;  1) postscript file containing graphs.
;
;  v1.0 - 06/27/2008 - Jeff McIntire
;*****************************************************************************

PRO RC5_OBC_view_req, infile

close,/all

restore,'general.TEB.sav'

;*****************************************************************************
;  Read in data.
;*****************************************************************************

nlines=File_Lines(infile)
ab=''
header=strarr(1)
inpath=strarr(1)		; Input path.
outpath=strarr(1)		; Output path.
rvs_in=strarr(1)
param_in=strarr(1)
rsrpath=strarr(1)
fnumber=fltarr(1)

openr,lun,infile,/Get_Lun	
readf,lun,header	
readf,lun,inpath
readf,lun,outpath
readf,lun,rvs_in
readf,lun,param_in
readf,lun,rsrpath
readf,lun,fnumber
Free_Lun,lun

a2=strsplit(outpath,'/',/Extract)
a3=N_ELEMENTS(a2)

set_multicolor,xsave,ysave,psave,chname,lcol,symbol,red,green,blue
set_plot,'ps'
outfname=outpath+a2[a3-3]+'.'+a2[a3-2]+'.'+a2[a3-1]+'.req.ps'
device,/landscape,filename=outfname,/color

infname=strarr(6)
infname[0]=outpath+a2[a3-3]+'.'+a2[a3-2]+'.'+a2[a3-1]+'.rad.sav'
infname[1]=outpath+a2[a3-3]+'.'+a2[a3-2]+'.'+a2[a3-1]+'.dn.sav'
infname[2]=outpath+a2[a3-3]+'.'+a2[a3-2]+'.'+a2[a3-1]+'.rg.sav'
infname[3]=outpath+a2[a3-3]+'.'+a2[a3-2]+'.'+a2[a3-1]+'.eff_rad.sav'
infname[4]=outpath+a2[a3-3]+'.'+a2[a3-2]+'.'+a2[a3-1]+'.temp.sav'
check=FILE_SEARCH(infname[0])
IF check EQ '' THEN BEGIN
	print,'No file is found.',infname[0]
	goto,skip_loop
ENDIF
FOR i=0, 4 DO BEGIN
	print,'Processing file:',infname[i]
	restore,infname[i]
ENDFOR

maxscans=100

D=findgen(ncollects)
E=findgen(3000)
F=findgen(maxscans*ncollects)

;*****************************************************************************
;  Determine the cutoff between warm-up and cool-down.
;*****************************************************************************

hcol=ncollects
FOR col=0, ncollects-2 DO BEGIN
	IF T_obc[0,col] GT T_obc[0,col+1] THEN hcol=hcol-1
ENDFOR

;*****************************************************************************
;  T_obc vs scan and collect (SRV0598 and SRV0654).
;*****************************************************************************

!p.multi=[0,1,1]

title='T_obc vs Scan and Collect for HAM 1 (SRV0598 and SRV0654)'
plot,F[*],xtit='Collect*scan',ytit='T_obc',yrange=[min(T_obc[0,*]),max(T_obc[0,*])],charsize=1.2,charthick=5,/nodata

FOR col=0, ncollects-1 DO BEGIN
	oplot,F[maxscans*col:maxscans*(col+1)-1],T_obc_2[0:maxscans-1,0,col],symsize=1,color=lcol[col],psym=symbol[col]
ENDFOR
XYOUTS,0.5,1.025,title,/normal,alignment=0.5,charsize=1.6,charthick=5

!p.multi=[0,3,3]
!p.position=0

title='T_obc vs Scan, HAM=1 (SRV0598 and SRV0654)'
FOR col=0, ncollects-1 DO BEGIN
	title1='Collect='+strtrim(string(col+1),2)

	ymax=max(T_obc_2[*,0,col])
	IF col LE hcol-1 THEN BEGIN
		plot,T_obc_2[*,0,col],xtit='Scan',ytit='T_obc',yrange=[ymax-0.05,ymax+0.05],title=title1, $
			charsize=1.2,charthick=5,/nodata

		oplot,T_obc_2[*,0,col],symsize=1,color=lcol[col],psym=symbol[col]
	ENDIF
ENDFOR
XYOUTS,0.5,1.025,title,/normal,alignment=0.5,charsize=1.6,charthick=5

!p.multi=[0,3,3]
!p.position=0

title='T_obc vs Scan, HAM=1 (SRV0598 and SRV0654)'
FOR col=0, ncollects-1 DO BEGIN
	title1='Collect='+strtrim(string(col+1),2)

	ymax=max(T_obc_2[*,0,col])
	IF col GT hcol-1 THEN BEGIN
		plot,T_obc_2[*,0,col],xtit='Scan',ytit='T_obc',yrange=[ymax-1.,ymax+1.],title=title1, $
			charsize=1.2,charthick=5,/nodata

		oplot,T_obc_2[*,0,col],symsize=1,color=lcol[col],psym=symbol[col]
	ENDIF
ENDFOR
XYOUTS,0.5,1.025,title,/normal,alignment=0.5,charsize=1.6,charthick=5

!p.multi=[0,2,1]
!p.position=0

FOR col=0, ncollects-1 DO BEGIN
	title='T_obc vs Scan: Collect='+strtrim(string(col+1),2)+' (SRV0598 and SRV0654)'
	FOR ih=0, nham-1 DO BEGIN
		title1='HAM='+strtrim(string(ih+1),2)

		ymax=max(T_obc_2[*,ih,col])
		IF col LE hcol-1 THEN BEGIN
			plot,T_obc_2[*,ih,col],xtit='Scan',ytit='T_obc',yrange=[ymax-0.05,ymax+0.05],title=title1, $
				charsize=1.2,charthick=5,/nodata

			oplot,T_obc_2[*,ih,col],symsize=1,color=lcol[col],psym=symbol[col]
		ENDIF
		IF col GT hcol-1 THEN BEGIN
			plot,T_obc_2[*,ih,col],xtit='Scan',ytit='T_obc',yrange=[ymax-1.,ymax+1.],title=title1, $
				charsize=1.2,charthick=5,/nodata

			oplot,T_obc_2[*,ih,col],symsize=1,color=lcol[col],psym=symbol[col]
		ENDIF
		XYOUTS,0.5,1.025,title,/normal,alignment=0.5,charsize=1.6,charthick=5
	ENDFOR
ENDFOR

;*****************************************************************************
;  Standard deviation of T_obc (SRV0095).
;*****************************************************************************

!p.multi=[0,1,1]
title='Standard Deviation of T_obc vs Collect (SRV0095)'

ymax=max(std_obc_temp[0,*])
plot,std_obc_temp[0,*],xtit='Collect',ytit='Standard Deviation of T_obc',xrange=[0,ncollects], $
  yrange=[0,ymax+0.04],charsize=1.2,charthick=5,/nodata
FOR i=0, 5 DO BEGIN
    oplot,D[0:hcol-1]+1,std_obc_temp[i,0:hcol-1],symsize=1,color=lcol[i],psym=symbol[i]
    oplot,[0,100],[0.03,0.03],color=lcol[1],linestyle=2,thick=1.5
ENDFOR
XYOUTS,0.5,1.025,title,/normal,alignment=0.5,charsize=1.6,charthick=5

;*****************************************************************************
;  Absolute Radiance Difference (SRV0545 and SRV0546).
;*****************************************************************************

FOR col=0, ncollects-1 DO BEGIN
	IF col GT 0 AND col LT ncollects-1 THEN BEGIN
		IF abs(T_bcs[col-1]-267) GT abs(T_bcs[col]-267) AND abs(T_bcs[col]-267) LT abs(T_bcs[col+1]-267) THEN BEGIN
			col_267=col
		ENDIF
		IF abs(T_bcs[col-1]-230) GT abs(T_bcs[col]-230) AND abs(T_bcs[col]-230) LT abs(T_bcs[col+1]-230) THEN BEGIN
			col_230=col
		ENDIF
		IF abs(T_bcs[col-1]-270) GT abs(T_bcs[col]-270) AND abs(T_bcs[col]-270) LT abs(T_bcs[col+1]-270) THEN BEGIN
			col_270=col
		ENDIF
		IF abs(T_bcs[col-1]-310) GT abs(T_bcs[col]-310) AND abs(T_bcs[col]-310) LT abs(T_bcs[col+1]-310) THEN BEGIN
			col_310=col
		ENDIF
	ENDIF
ENDFOR
col_190=0
col_340=ncollects-1
scol=[col_190,col_230,col_270,col_310,col_340]

ARD_req_I=[5,2.5]
ARD_req_M=[[0,0,12.3,2.1,1.6,1.6],[7.0,5.7,2.4,0.60,.60,.60],[0.7,0.7,0.6,0.4,0.4,0.4], $
	[0.7,0.7,0.4,0.4,0.4,0.4],[0.7,0.7,0.5,0.4,0.4,0.4]]
collect_name=['190','230','270','310','340']

!p.multi=[0,3,3]
!p.position=0

G=findgen(detectors_TEB[0])
title1='Absolute Radiance Difference (SRV0546)'
FOR b=0, nbands_TEB-1 DO BEGIN
	IF band_type_TEB[b] EQ 0 THEN continue

	title1='Absolute Radiance Difference (SRV0546)'
	title2=band_name_TEB[b]+', T_F6=267'

	plot,G[*]+1,abs(ARD_obc[col_267,b,0,*,0,1]),xtit='Detectors',ytit='ARD',yrange=[0,ARD_req_I[b]+2], $
		xrange=[0,detectors_TEB[b]],title=title2,charsize=1.2,charthick=5,/nodata

	oplot,G[*]+1,abs(ARD_obc[col_267,b,0,*,0,1]),symsize=1,color=lcol[b],psym=symbol[b]
	oplot,G[*]+1,abs(ARD_obc[col_267,b,0,*,0,1]),color=lcol[b]

	oplot,[0,50],[ARD_req_I[b],ARD_req_I[b]],symsize=1,color=lcol[1],linestyle=2,thick=1.5
	oplot,[0,50],[-ARD_req_I[b],-ARD_req_I[b]],symsize=1,color=lcol[1],linestyle=2,thick=1.5
ENDFOR

G=findgen(detectors_TEB[2])
FOR b=0, nbands_TEB-1 DO BEGIN
	IF band_type_TEB[b] EQ 1 THEN continue

	FOR j=0, 4 DO BEGIN
		IF j LE 1 OR j EQ 4 THEN continue

		title2=band_name_TEB[b]+', T_F6='+collect_name[j]

		ymax=max(ARD_obc[scol[j],b,0,0:detectors_TEB[b]-1,0,1])
		IF ymax LT ARD_req_M[b-2,j] THEN ymax=ARD_req_M[b-2,j]*(1.1)

		plot,G[*]+1,abs(ARD_obc[scol[j],b,0,*,0,1]),xtit='Detectors',ytit='ARD',yrange=[0,ymax], $
			xrange=[0,detectors_TEB[b]],title=title2,charsize=1.2,charthick=5,/nodata

		oplot,G[*]+1,abs(ARD_obc[scol[j],b,0,*,0,1]),symsize=1,color=lcol[b],psym=symbol[b]
		oplot,G[*]+1,abs(ARD_obc[scol[j],b,0,*,0,1]),color=lcol[b]

		oplot,[0,50],[ARD_req_M[b-2,j],ARD_req_M[b-2,j]],symsize=1,color=lcol[1],linestyle=2,thick=1.5
	ENDFOR
ENDFOR
XYOUTS,0.5,1.025,title1,/normal,alignment=0.5,charsize=1.6,charthick=5

G=findgen(detectors_TEB[0])
FOR b=0, nbands_TEB-1 DO BEGIN
	IF band_type_TEB[b] EQ 0 THEN continue
	!p.multi=[0,1,1]
	!p.position=0

	name=[band_name_TEB[b]+' SS1 HAM A',band_name_TEB[b]+' SS1 HAM B', $
		band_name_TEB[b]+' SS2 HAM A',band_name_TEB[b]+' SS2 HAM B','Spec']

	title1='Absolute Radiance Difference (SRV0546)'
	title2=band_name_TEB[b]+', T=267 (quadratic poly_fit)'

	plot,G[*]+1,abs(ARD_obc[col_267,b,0,*,0,1]),xtit='Detectors',ytit='ARD',yrange=[0,ARD_req_I[b]+2], $
		xrange=[0,detectors_TEB[b]],title=title2,charsize=1.2,charthick=5,/nodata

	m=0
	FOR ih=0, nham-1 DO BEGIN
		FOR is=0, subsets_TEB[b]-1 DO BEGIN
			oplot,G[*]+1,abs(ARD_obc[col_267,b,ih,*,is,1]),symsize=1,color=lcol[m],psym=symbol[m]
			oplot,G[*]+1,abs(ARD_obc[col_267,b,ih,*,is,1]),color=lcol[m]
			m=m+1
		ENDFOR
	ENDFOR
	oplot,[0,50],[ARD_req_I[b],ARD_req_I[b]],symsize=1,color=lcol[1],linestyle=2,thick=1.5
	oplot,[0,50],[-ARD_req_I[b],-ARD_req_I[b]],symsize=1,color=lcol[1],linestyle=2,thick=1.5

	XYOUTS,0.5,1.025,title1,/normal,alignment=0.5,charsize=1.6,charthick=5
	legend,name,linestyle=[0,0,0,0,1],colors=[0,1,2,3,1],thick=2,charthick=5,/right_legend
ENDFOR

G=findgen(detectors_TEB[2])
FOR b=0, nbands_TEB-1 DO BEGIN
	IF band_type_TEB[b] EQ 1 THEN continue
	!p.multi=[0,1,1]
	!p.position=0

	name=[band_name_TEB[b]+' HAM A',band_name_TEB[b]+' HAM B','Spec']

	k=0
	FOR j=0, 4 DO BEGIN
		title1='Absolute Radiance Difference (SRV0545)'
		title2=band_name_TEB[b]+', T='+collect_name[j]+' (quadratic poly_fit)'

		IF j LT 1 OR j EQ 4 THEN continue 

		ymax=max(ARD_obc[scol[j],b,0,0:detectors_TEB[b]-1,0,1])
		IF ymax LT ARD_req_M[b-2,j] THEN ymax=ARD_req_M[b-2,j]*(1.1)

		plot,G[*]+1,abs(ARD_obc[scol[j],b,0,*,0,1]),xtit='Detectors',ytit='ARD',yrange=[0,ymax], $
			xrange=[0,detectors_TEB[b]],title=title2,charsize=1.2,charthick=5,/nodata

		k=0
		FOR ih=0, nham-1 DO BEGIN
			FOR is=0, subsets_TEB[b]-1 DO BEGIN
				oplot,G[*]+1,abs(ARD_obc[scol[j],b,ih,*,is,1]),symsize=1,color=lcol[k],psym=symbol[k]
				oplot,G[*]+1,abs(ARD_obc[scol[j],b,ih,*,is,1]),color=lcol[k]
				k=k+1
			ENDFOR
		ENDFOR
		oplot,[0,50],[ARD_req_M[b-2,j],ARD_req_M[b-2,j]],symsize=1,color=lcol[1],linestyle=2,thick=1.5

		XYOUTS,0.5,1.025,title1,/normal,alignment=0.5,charsize=1.6,charthick=5
		legend,name,linestyle=[0,0,1],colors=[0,1,1],thick=2,charthick=5,/right_legend
	ENDFOR
ENDFOR

device,/close

skip_loop:

END
